%                     Fig1_9andTable1_1.m
%
%              calls funct.m  and jacobi.m
%
%
%           Reliability problem in Example 1.25  (see also Maymon 
%                                           "Some engineering applications
%                                           in random vibrations and random
%                                           structures" 1998)
%  Input data of Example 1.25:
%    Sy  - bending moment   (Weibull distribution)
%    T  - Axial lod              (normal)
%    A  - area of cross-section  (normal)
%    
%    
%  Following Rackwitz-Fiessler (Table A.1 in Appendix A) procedure [1978],
%           the reliability index 'beta' is obtained 
%                   by an iterative process 
%             and finally the probability of failure
%
%__________________________________________________________________________
%
%   The program then uses Equation (2.46) to get samples of Bm
%           
%     The program outputs results on reliability index 
%          & probability of failure given in Table 2.1 

%
% Given the correlated non-normals, this program transforms the 2 x 1
%     RV vector tO correlated standard Normal RVs first
%            and then TO uncorrelated standard Normal RVs
%     The Cholesky decomposition of covariance matrix is used
%                   in the last phase.
%
%   param : given paramters of pdfs of the original RV vector - x
%   cov : n x 1 given vector of Coefficient of Variaiton of original (basic)
%                      random variables
%   sigma  : n x 1 vector of given (input) standard deviation of x
%   stdev : n x 1 vector of standard deviation of RV x - values after
%                  computation
%   rho : given (input) correlation coefficient matrix of x
%   rhoNat   : nxn correlation matrix of equivalen normal variables in Nataf
%              model
%   drho : values of 'rho' used to stand for the correlations of the
%         standard correlated Normal RVs
%   dx :(n + 1)x1 vector that arises in the Newton-Raphson
%   determination of design point

%   mu    : nx1 vector of specified mean of x
%   n : size of x
%   m : nx1 vector of computed mean of x (for verification purpose)
%   This program also gives plots of pdfs for x
%   It also plots normal pdf curve having same mean and standard deviation
%----------------------------------------------------------------------------
clc;
clear all;
close all;
%
       pi=3.14159265;
       pi2=2*pi;
%
% Input data on mean and standard deviations of the three RVs                   
      mSy=2500.0;
      mT=4166.0;
      mA=2.0;       
      sigSy=75.0;
      sigT=125;
      sigA=0.04;
%
% parameters of the Weibull distribution (of Sy)
%        Equation (1.173)   (see also Maymon 1998)
      alpha=8.18211*10^(-144);
      betaa=42.03862;
% 
%
%  
      my1=mSy;
      my2=mT;
      my3=mA;
      sigy1 = sigSy;
      sigy2 = sigT;
      sigy3 = sigA;  
%      
% plots of marginal distributions
% Sy is Weibul
      alpha=8.18211*10^(-144);
      betaa=42.03862;
      yy(1)=1000.0;
%      
      Wpdf(1)=alpha*betaa*yy(1)^(betaa-1)*exp(-alpha*yy(1)^betaa);
%   
%   To plot Figure 1.9a - Weibill pdf of Sy
      sumWpdf=0.0;
      for i=2:201
        yy(i)=yy(i-1)+10;
        Wpdf(i)=alpha*betaa*yy(i)^(betaa-1)*exp(-alpha*yy(i)^betaa);      
        sumWpdf=sumWpdf+Wpdf(i)*10;
      end
       'area under the pdf curve of RV- Sy'
      sumWpdf    % for checking the area under the Weibull pdf of Sy 
      figure(1)             %  Figure 1.9a
      plot(yy,Wpdf,'black')
%    
%   
%   axial load T - normal
%       To plot Figure 1.9b - normal pdf of T, the axial load
      zz(1)=2000.0;          
      Tpdf(1)=normpdf(zz(1),mT,sigT);
      sumTpdf=0.0;  
      for i=2:401   
       zz(i)=zz(i-1)+10;          
       Tpdf(i)=normpdf(zz(i),mT,sigT);      
       sumTpdf=sumTpdf+Tpdf(i)*10;
      end
         'area under the pdf curve of RV- T'
      sumTpdf % for checking the area under the normal pdf of T, axial load 
      figure(2)     %  Figure 1.9b
      plot(zz,Tpdf,'black') 
%
%
%  area of cross-section, A - normal
%       To plot Figure 1.9c - normal pdf of A, the area of cross-section
      ww(1)=1.0;          
      Apdf(1)=normpdf(ww(1),mA,sigA);
      sumApdf=0.0;
      for i=2:200
        ww(i)=ww(i-1)+0.01;          
        Apdf(i)=normpdf(ww(i),mA,sigA);      
        sumApdf=sumApdf+Apdf(i)*0.01;
      end
   'area under the pdf curve of RV- A'
      sumApdf    % for checking the area under the normal pdf of A, area of cross-section
      figure(3)       %  Figure 1.9c
      plot(ww,Apdf,'black')
 %
 %    
 %    
 %________________________________________________________________________
 %
 %    reliability index  :  beta= sqrt(z1^2 + z2^2 + z3^2)
 %     
 %    Target : minimize beta 
 %
 %       Constraint :   Limit state function: g =  Sy-T/A 
 %          Limit state function in terms of z variable vector:
 %                  g =  (sigSy*z1 +mSy)-(sigT*z2 +mT)/(sigA*z3 +mA)=0
 %
 %              Minimization of the objective function by Lagrangian
 %                      multiplier method
 % 
 %    Objective function :  (z1^2 + z2^2 + z3^2) 
 %                   - lamda* {(sigSy*z1 +mSy)-(sigT*z2 +mT)/(sigA*z3 +mA)}
 %
  %      differentiating wrt z variables and lamda:
 %      (1) wrt lamda: (sigSy*z1 +mSy)*(sigA*z3 +mA)-(sigT*z2 +mT)=0
 
 %      (2) wrt z1:  2*z1 - lamda*sigSy  = 0
 %                              This gives lamda=2*z1/sigSy
 %
 %      (3) wrt z2:  2*z2*(sigA*z3 +mA) + lamda*sigT=0
 %          
 %      (4) wrt z3:  2*z3*(sigA*z3+mA)^2 - lamda* (sigT*z2 +mT)*sigA=0
 %
  %
 %          Substitute 'lamda' from Equation (2) in Equations (3) and (4)
 %              and solve the resulting three non-linear Equations
 %             (including Equation 1)to compute z1,z2 & z3 
 %                      by Newton-raphson method
 %_______________________________________________________________________
 %
 % Start of the iteration procedure ( see Table A.1 in Appendix A)
  itr=5;  %    No. of iterations
 %
 %   Initialization for the standard normal variables z={z1,z2,z3}
 %      This implies assumption of equivalent z RVs instead of 
                                  %  Non-normal input RVs x={Sy,T,A} 
%                                                        
     x(1)=0.0;
     x(2)=0.0;
     x(3)=0.0;   
%
      tol=0.0001;
%
      for outer_iteration=1:itr   
        outer_iteration        % counter for outer iteration for improving 'beta'
                                 % see steps 2-5 in table A.1 (Appendix A)
%
      for inner_iteration=1:itr     
        inner_iteration;    % counter for iteration counter for Newton-raphson method
                                % to obtain approximate 'beta'
%
%         Solve by Newton-raphson method
%
%  nonlinear functions of the final three equations resulting from 
%           Equations (1) to (4) above
        fn= funct(x,sigy1,sigy2,sigy3,my1,my2,my3); 
%
%            [rjac] :     Jacobian matrix                                                            
        rjac=jacobi(x,sigy1,sigy2,sigy3,my1,my2,my3);
%
        for i=1:3
          sum=0.0;
         for j=1:3
            sum=sum-rjac(i,j)*fn(j);   
         end
          h(i)=sum;
        end
%
       for i=1:3
          x(i)=x(i)+h(i);  %new z vector by iterative Newton-Raphson method
       end
%       
      end  % end of inner iterations
%      
      beta=sqrt( x(1)*x(1) + x(2)*x(2) + x(3)*x(3) ) % reliability index
                                                     % at the end of each
                                                     % inner iteration
%
% chang to original RVs 
        y(1)=sigy1*x(1)+my1;    % new Sy variable
        y(2)=sigy2*x(2)+my2;    % new T variable
        y(3)=sigy3*x(3)+my3;    % new A veriable
%
fprintf(1, 'Sy = %f, T = %f, A = %f \n\n', y(1), y(2), y(3));    
%      
%  Y={Sy, T, A} at the end of each inner iteration  (recorded in Table 1.1)        
%         
%        
%  to find equivalent normal variables corresonding to new Sy, T and A
%
%      Only for Sy which is Weibull, we need to compute  
%                               equivalent normal variable
%
% by Nataf transformation (Equation 1.168)
y1=(y(1))^(betaa);
Fy=1-exp(-alpha*y1);
tmp=norminv(Fy);
weidenf=alpha*betaa*(y(1)^(betaa-1))*exp(-alpha*y(1)^betaa);
sigynew=normpdf(tmp)/weidenf;
sigy1=sigynew ;              % standard deviation for equivalent normal
my1=y(1)-sigy1*tmp;          % mean for equvalent normal
%
%
x(1)=(y(1)-my1)/sigy1  ;    % equivalent standard normal for 'Sy'
x(2)=(y(2)-my2)/sigy2  ;    % equivalent standard normal for 'T'
x(3)=(y(3)-my3)/sigy3  ;    % equivalent standard normal for 'A'
%
%
      end   % end of outer iterations
     
'Final result at the end of iterations'    
%                 
pf=normcdf(-beta);       %  probability of failure
%
% final 'beta' - reliability index and 'pf' probability of failure
fprintf(1, 'relaibility index = %f, prob of failure = %f \n\n', beta, pf);   
%
 % Sy,  T   and A  at the failure point
fprintf(1, 'Sy = %f, T = %f, A = %f \n\n', y(1), y(2), y(3));                                            
                    


