%                       Example5_10_Fig5_17a.m  
%
%                Basic LTL methods (Section 5.9.1)  with dt =0.001
%
%           Solution to the Duffing oscillator in Equation 5.178
%                      (refer to Saha and Roy 2007)
%
%                   additive noise + determinsitic harmonic force
%
%                           Phase plane curve
%
%       Figure 5.17a:  E[x(t)] - E[dx(t)] phase plane curve 
%                 by basic LTL scheme (Equation 5.170) 
%__________________________________________________________________________
%
clc
clear all
close all
%
pi=3.14159265;
pi2=2*pi;
%
T=20.0;   % total time of integration
%
% parameters of the 5- parameter Duffing oscillator [Saha and Roy 2007]
  e1=0.25;
  e2=1.0;
  e3=41;    % harmonic excitation
  e4=0.5;   % only additive noise 
  e5=0.0;   % no multiplicative noise
% the above parmeters are changed to non-dimensional time, tou=2*pi*t
  c1=pi2*e1;
  c2=4*pi*pi*e2;
  c3=4*pi*pi*e3;
  c4=e4;
  c5=e5;
%
N=1000;   % no. of samples
%
%iniial conditions
x0=0.0;
dx0=0.0;
%
dt1=0.001;
h=dt1;
tm=0:dt1:T;  % time variable
N1=numel(tm);
%
%
for j=1:N
j % sample counter
xwMSI(1,j)=x0;
dxwMSI(1,j)=dx0;
%
%  start for solution by basic LTL sceme in Equation (5.170) 
%                                   with MSIs
  y1old=x0;
  y2old=dx0;
%
  for k=2:N1
%
  u1=randn;
  u2=randn;
%  
  I1 = sqrt(dt1)*u1;
  I2 = sqrt(dt1)*u2;
%
% start of Newton-Raphson scheme to solve one nonlinear algebraic 
%           equation of displacement variable
    y1a=y1old;
    y2a=y2old;
%
    err=1;
%
       while err>1e-5
      y1a_1=y1a;
      y2a_1=y2a;
%
      phi11 = 1 - 0.5*h*h*c2;
      phi11 = phi11 + (1/6.0)*h*h*h*c1*c2;
%
      phi12 = h - 0.5*h*h*c1;
      phi12 = phi12 + (1/6.0)*h*h*h*(c1*c1-c2);
%
      phi21 = -c2*h;
      phi21 = phi21+ 0.5*h*h*c1*c2;
      phi21 = phi21+ (1/6.0)*h*h*h*(c2*c2-c1*c1*c2);
%
      phi22 = 1 - c1*h + 0.5*h*h*(c1*c1-c2);
      phi22 = phi22 + (1/6.0)*h*h*h*(2*c1*c2-c1*c1*c1);
%
      ihp11 = 1 - 0.5*h*h*c2;
      ihp11 = phi11 - (1/6.0)*h*h*h*c1*c2;
%
      ihp12 = -h - 0.5*h*h*c1;
      ihp12 = phi12 - (1/6.0)*h*h*h*(c1*c1-c2);
%
      ihp21 = c2*h;
      ihp21 = ihp21+ 0.5*h*h*c1*c2;
      ihp21 = ihp21- (1/6.0)*h*h*h*(c2*c2-c1*c1*c2);
%
      ihp22 = 1 + c1*h + 0.5*h*h*(c1*c1-c2);
      ihp22 = phi22 - (1/6.0)*h*h*h*(2*c1*c2-c1*c1*c1);
%
      fun1 = y1a - phi11*y1old - phi12*y2old;
      f1 = (1/(2*pi))*(sin(2*pi*tm(k)) - sin(2*pi*tm(k-1)));
      f2 = c2*y1a^3*h;
%
      fun1=fun1 - phi11*ihp12*(c3*f1-f2);
      fun1=fun1 - phi12*ihp22*(c3*f1-f2);
      fun1=fun1 - c4*(phi11*ihp12*I1 + phi12*ihp22*I2);
      fun1=fun1 - c5*y1a*(phi11*ihp12*I1 + phi12*ihp22*I2);
%
      df2=3*c2*y1a*y1a*h;
      df1 = 1.0 + df2*(phi11*phi12+phi12*phi22);
      df1 = df1 - c5*(phi11*ihp12*I1 + phi12*ihp22*I2);
%
      y1new=y1a-fun1/df1;
%
      ff2 = c2*y1new^3*h;
      fun2 = phi21*y1old + phi22*y2old;
      fun2 = fun2 + phi21*ihp12*(c3*f1 - ff2);
      fun2 = fun2 + phi22*ihp22*(c3*f1 - ff2);
      fun2 = fun2 + c4*(phi21*ihp12*I1 + phi22*ihp22*I2);
      fun2 = fun2 + c5*y1new*(phi21*ihp12*I1 + phi22*ihp22*I2);
      y2new=fun2;
%
      y1a_2=y1new;
      y2a_2=y2new;
%
      err=(y1a_2-y1a_1)^2+(y2a_2-y2a_1)^2;
      err=sqrt(err);
%
      y1a=y1new;
      y2a=y2new;
%
       end   %  start of Newton-Raphson scheme
%
    y2old=y2new;
    y1old=y1new;
%
      xwMSI(k,j)=y1new;
      dxwMSI(k,j)=y2new;
%
  end    % end of time integration     k=1:N1
%
end    % of N samples
%---------------------------------------------------------------
%
% moment time histories
for k=1:N1
    xwM(k)=0.0;
    dxwM(k)=0.0;
%
    for j=1:N
    xwM(k)= xwM(k)+xwMSI(k,j)/N;
    dxwM(k)= dxwM(k)+dxwMSI(k,j)/N;
    end
end
%
% plotting
% figure(2)   % not plotted 
% plot(tm,xwM),hold on 
% figure(3)    % not plotted
% plot(tm,dxwM),hold on
%
figure(1)     % Figure 5.17a of basic LTL scheme
plot(xwM,dxwM,'black')   % phase plane curve 


