%                       Example5_10_Fig5_17b.m  
%
%            Higher order LTL method (Section 5.9.1) with dt=0.001
%
%           Solution to the Duffing oscillator in Equation 5.178
%                      (refer to Saha and Roy 2007)
%
%                   additive noise + determinsitic harmonic force
%
%                           Phase plane curve
%
%       Figure 5.17b:  E[x(t)] - E[dx(t)] phase plane curve 
%                 by Higher order LTL scheme (Equation 5.177) 
%__________________________________________________________________________
%
clc
clear all
close all
%
pi=3.14159265;
pi2=2*pi;
%
T=20.0; % total time of integration
%
% parameters of the 5- parameter Duffing oscillator [Saha and Roy 2007]
  e1=0.25;   
  e2=1.0;    
  e3=41.0;   % harmonic excitation
  e4=0.5;    % only additive noise 
  e5=0.0;    % no multiplicative noise
% the above parmeters are changed to non-dimensional time, tou=2*pi*t
  c1=pi2*e1;
  c2=4*pi*pi*e2;
  c3=4*pi*pi*e3;
  c4=e4*4*pi*pi;
  c5=e5*4*pi*pi;
%
N=1000;  % no. of samples
%
%iniial conditions
x0=0.0;
dx0=0.0;
%
dt1=0.001; 
h=dt1;
tm=0:dt1:T;   % time variable
N1=numel(tm);
%
for k=1:N1
 xwM(k)=0.0;    
 dxwM(k)=0.0;
end 
%
for j=1:N
j % sample counter
xwMSI(1,j)=x0;
dxwMSI(1,j)=dx0;
%
%  start for solution by higher order LTL sceme in Equation (5.177) 
%                                   with MSIs
  y1old=x0;
  y2old=dx0;
%
  for k=2:N1

  u1=randn;
  u2=randn;
  u3=randn;
  u4=randn;
%
  db1=sqrt(dt1)*u3;
  db2=sqrt(dt1)*u4;
%
  I1 = sqrt(dt1)*u1;
  I2 = sqrt(dt1)*u2;
%
  I20=0.5*dt1*(db1+db2/sqrt(3));
  I02=0.5*dt1*(db1-db2/sqrt(3));
%
% start of Newton-Raphson scheme to solve two nonlinear algebraic 
%           equations of displacement variable
    y1a=y1old;
    y2a=y2old;
%
    err=1;
%
    while err>1e-5
      y1a_1=y1a;
      y2a_1=y2a;
%
      phi11 = 1 - 0.5*h*h*c2;
      phi11 = phi11 + (1/6.0)*h*h*h*c1*c2;
%
      phi12 = h - 0.5*h*h*c1;
      phi12 = phi12 + (1/6.0)*h*h*h*(c1*c1-c2);
%
      phi21 = -c2*h;
      phi21 = phi21+ 0.5*h*h*c1*c2;
      phi21 = phi21+ (1/6.0)*h*h*h*(c2*c2-c1*c1*c2);
%
      phi22 = 1 - c1*h + 0.5*h*h*(c1*c1-c2);
      phi22 = phi22 + (1/6.0)*h*h*h*(2*c1*c2-c1*c1*c1);
%
      ihp11 = 1 - 0.5*h*h*c2;
      ihp11 = phi11 - (1/6.0)*h*h*h*c1*c2;
%
      ihp12 = -h - 0.5*h*h*c1;
      ihp12 = phi12 - (1/6.0)*h*h*h*(c1*c1-c2);
%
      ihp21 = c2*h;
      ihp21 = ihp21+ 0.5*h*h*c1*c2;
      ihp21 = ihp21- (1/6.0)*h*h*h*(c2*c2-c1*c1*c2);
%
      ihp22 = 1 + c1*h + 0.5*h*h*(c1*c1-c2);
      ihp22 = phi22 - (1/6.0)*h*h*h*(2*c1*c2-c1*c1*c1);
%
      fun1 = y1a - phi11*y1old - phi12*y2old;
      f1 = (1/(2*pi))*(sin(2*pi*tm(k)) - sin(2*pi*tm(k-1)));
%
      f2 = c2*(y1a^3*h - 3*y1a*y1a*y2a*h*h*0.5);
      f2 = f2 + c2*(3*y1a*y2a*y2a*h*h*h*(1/6.0));
      f2 = f2 - c2*(y2a*y2a*y2a*h*h*h*h*(1/24.0));
      fun1=fun1 - phi11*ihp12*(c3*f1-f2);
      fun1=fun1 - phi12*ihp22*(c3*f1-f2);
      fun1=fun1 - c4*(phi11*ihp12*I1 + phi12*ihp22*I1);
      fun1=fun1 - c5*y1a*(phi11*ihp12*I2 + phi12*ihp22*I2);
      fun1=fun1 - c5*y2a*(phi11*ihp12*I02 + phi12*ihp22*I02);
%
      fun2 = y2a - phi21*y1old - phi22*y2old;
      fun2 = fun2 - phi21*ihp12*(c3*f1 - f2);
      fun2 = fun2 - phi22*ihp22*(c3*f1 - f2);
      fun2 = fun2 - c4*(phi21*ihp12*I1 + phi22*ihp22*I1);
      fun2 = fun2 - c5*y1a*(phi21*ihp12*I2 + phi22*ihp22*I2);
      fun2 = fun2 - c5*y2a*(phi21*ihp12*I02 + phi22*ihp22*I02);
%
      f2d1 = 3*c2*y1a*y1a*h;
      f2d1 = f2d1 - 6*c2*y1a*y2a*h*h*0.5;
      f2d1 = f2d1 + 3*c2*y2a*y2a*h*h*h*(1/6.0);
      df11 = 1.0 + f2d1*(phi11*ihp12+phi12*ihp22);
      df11 = df11 - c5*(phi11*ihp12*I2 + phi12*ihp22*I2);
%
      f2d2= -c2*3*y1a*y1a*h*h*0.5;
      f2d2= f2d2+c2*(6*y1a*y2a*h*h*h*(1/6.0));
%
      f2d2= f2d2 - c2*(3*y2a*y2a*h*h*h*h*(1/24.0));
      df12 = f2d2 - c5*(phi11*ihp12*I02 + phi12*ihp22*I02);
%
      df21 = f2d1*(phi21*ihp12+phi22*ihp22);
      df21 = df21 - c5*(phi21*ihp12*I2 + phi22*ihp22*I2); ;
%
      df22 = 1.0 + f2d2 - c5*(phi21*ihp12*I02 + phi22*ihp22*I02); ;
%
      det=df11*df22-df12*df21;
      j11=df22/det;
      j12=-df12/det;
      j21=-df21/det;
      j22=df11/det;
%
      dy1=j11*fun1+j12*fun2;
      dy2=j21*fun1+j22*fun2;
%
      y1new=y1a-dy1;
      y2new=y2a-dy2;
%
      y1a_2=y1new;
      y2a_2=y2new;
%
      err=(y1a_2-y1a_1)^2+(y2a_2-y2a_1)^2;
      err=sqrt(err);
%
      y1a=y1new;
      y2a=y2new;
%
    end   % end of Newton-Raphson scheme
%
    y2old=y2new;
    y1old=y1new;
%      
      xwM(k)=xwM(k)+y1new/N;    
      dxwM(k)=dxwM(k)+y2new/N;   
%
  end    % end of integration     k=1:N1
%
end    % of N samples
%---------------------------------------------------------------
% plotting
% figure(2)   % not plotted 
% plot(tm,xwM,'black'),hold on 
% figure(3)    % not plotted
% plot(tm,dxwM,'black'),hold on
%
figure(1),hold on  % Figure 5.17b of higher order LTL scheme with dt =0.001
plot(xwM,dxwM,'black')   % phase plane curve 

