%                               Example5_2_Fig5_6.m  
%
%                  Duffing oscillator (Equation 5.40)  in Example 5.2
%
%                    solution by the implicit EM method
%                      and comparison with explicit method
%
%               Figure 5.6a - E[x(t)]  - displacement
%           and Figure 5.6b - E[dx(t)] - velocity
%
%  Each figure contains 1) Explicit EM solution with dt=0.05  (Equation 5.36)
%                       2) Implicit EM solution with dt=0.05  (Equation 5.43)
%                       3) Implicit EM solution with dt=0.0001(Equation 5.43)
%__________________________________________________________________________
%
clc
clear all
close all
%
%
pi=3.14159265;
%
% %parameters of the Duffing oscillator - Equation (5.40)
c=4.0;
k=100.0;
alpha=100.0;
P=4.0;
sigma=0.01;
omega=2*pi; % frequency of the harmonic force
%
%
T=4.0; % total time of integration
%
%
N=100; % no. of samples in the MC run
%
%
%
%__________________________________________________________________________
%                unstable solution by  Explicit EM Run with dt=0.05
%
%initial conditions
x0=0.0;
dx0=0.0;
%
%
%     
dt=0.05;
t=0:dt:T;
%
%
for j=1:N
  x1old=x0;
  x2old=dx0;
  x(1)=x1old;
  dx(1)=x2old;
%
% numerical solution by Explicit EM method for Duffing oscillator  
  for i=1:numel(t)-1
    force=P*cos(omega*t(i));
    dB=sqrt(dt)*randn;
    x1new=x1old+x2old*dt;
    x2new=x2old+(-c*x2old-k*x1old-alpha*x1old^3+force)*dt+sigma*dB;
    dx(i+1,j)=x2new;
    x(i+1,j)=x1new;
    x1old=x1new;
    x2old=x2new;
  end
%
end
%
%
for i=1:numel(t)
  meanx(i)=mean(x(i,:));
  meandx(i)=mean(dx(i,:));
end
%plotting for EXplicit EM with dt=0.05
figure(1)
plot(t,meanx,'black'), hold on   % E[x(t)]
figure(2)
plot(t,meandx,'black'), hold on   % E[dx(t)]
%
%
%
%
%
%__________________________________________________________________________
%                    stable solution by implicit EM for dt =0.05
%
for j=1:N
% initial condition
y1old=x0;
y2old=dx0;
%
  for i=1:numel(t)-1
    force=P*cos(omega*t(i));
    dB=sqrt(dt)*randn;
%
    y2a=y2old;
    err=1;
%
% 
% numerical solution by Implicit EM method for Duffing oscillator  
    while err>1e-5
      y2a_1=y2a;
      fun=y2a-y2old-(-c*y2a-k*(y1old+y2a*dt)-alpha*(y1old+y2a*dt)^3+force)*dt-sigma*dB;
      dfun=1-(-c-k*dt-alpha*dt*3*(y1old+y2a*dt)^2)*dt;
      y2new=y2a-fun/dfun;
      y2a_2=y2new;
      err=abs(y2a_2-y2a_1);
      y2a=y2new;
    end
%
    y1new=y1old+y2new*dt;
%
    dy(i+1,j)=y2new;
    y(i+1,j)=y1new;
%
    y2old=y2new;
    y1old=y1new;
%
  end
%
end
%
for i=1:numel(t)
  meany(i)=mean(y(i,:));
  meandy(i)=mean(dy(i,:));
end
%plotting for Implicit EM with dt=0.05
figure(1)
plot(t,meany,'--black')   % E[x(t)]
figure(2)   
plot(t,meandy,'--black')  % E[dx(t)]
%
%
%
%
%
%__________________________________________________________________________
%                  (true soluton) stable solution by implicit EM for dt =0.0001
%        
dt=0.0001;
t=0:dt:T;
%
for j=1:N
    %
%initial conditions
z1old=x0;
z2old=dx0;
%
  for i=1:numel(t)-1
    force=P*cos(omega*t(i));
    dB=sqrt(dt)*randn;
%
    z2a=z2old;
    err=1;
% 
% numerical solution by Implicit EM method for Duffing oscillator  
    while err>1e-5
      z2a_1=z2a;
      fun=z2a-z2old-(-c*z2a-k*(z1old+z2a*dt)-alpha*(z1old+z2a*dt)^3+force)*dt-sigma*dB;
      dfun=1-(-c-k*dt-alpha*dt*3*(z1old+z2a*dt)^2)*dt;
      z2new=z2a-fun/dfun;
      z2a_2=z2new;
      err=abs(z2a_2-z2a_1);
      z2a=z2new;
    end
%
    z1new=z1old+z2new*dt;
%
    dz(i+1,j)=z2new;
    z(i+1,j)=z1new;
%
    z2old=z2new;
    z1old=z1new;
%
  end
%
end
%
for i=1:numel(t)
  meanz(i)=mean(z(i,:));
  meandz(i)=mean(dz(i,:));
end
%%plotting for Implicit EM with dt=0.0001
figure(1)
plot(t,meanz,'black')  % E[x(t)]
figure(2)
plot(t,meandz,'black')  % E[dx(t)]

