%                         Example5_5_Fig5_10.m  
%
%                         Stochastic Newmark method  
%                         (Equations 5.116 & 120)
%               
%             solution for Duffing oscillator  (same as of Example 5.2) (see Equation 5.40)
%
%                    Plots for different alpha and Beta
%                      (newmark implicit parameters)
%
%        Figure 5.10a contains:  1) solution for E[x(t)] by stochastic
%                                         Newmark method (Equation 5.123a)
%        Figure 5.10b contains:  2) solution for E[dx(t)] by stochastic
%                                         Newmark method (Equation 5.123b)
%
%           
%
%__________________________________________________________________________
%
clc
clear all
close all
%
%
pi=3.14159265;
%
%
%  parameters of the Duffing oscillator (Equation 5.40)
  c=4.0;
  k=100.0;
  gama=100.0;
  omega=2*pi;   % frequency of harmonic excitation
  sigma=0.01;   % noise intensity 
  P=4.0;   % amplitude of the harmonic excitation
%
%
%
N=1000;    % n. of samples
%
%
%------------------------------------------------------------------------
%               Solution by Equation (5.116 & 120) - Stochastic Newmark method
T=5;
dt1=0.05;
h=dt1;
tm=0:dt1:T;
N1=numel(tm)
%
%
%
%  Newmark implicit parameters
% alpha=0.5;
% beta=0.5;
% %
% initial conditions
x0=0.0;
dx0=0.0;
%
%
%
%
for k=1:4
    if(k==1)
        alpha=0.5;    
        beta=0.5;
    elseif (k==2)
        alpha=0.5;
        beta=0.25;
    elseif (k==3)
        alpha=0.25;
        beta=0.5;
    elseif (k==4)
        alpha=0.75;
        beta=0.5;
    end
    
 for j=1:N
  [k  j]   % alphaandbeta counter and sample counter  
 %
  y1old=x0;
  y2old=dx0;
  y(1)=y1old;
  dy(1)=y2old;
%
%
%
%
%  start of integration
  for i=2:N1
%
  u1=randn;
  u2=randn;
  u3=randn;
%
  db1=sqrt(dt1)*u2;
  db2=sqrt(dt1)*u3;
%
  db=sqrt(dt1)*u1;
%
  I10=0.5*dt1*sqrt(dt1)*(db1+db2/sqrt(3));
  I01=0.5*dt1*sqrt(dt1)*(db1-db2/sqrt(3));
%
    y1a=y1old;
    y2a=y2old;
%
    err=1;
%
    while err>1e-5
      y1a_1=y1a;
      y2a_1=y2a;
%
      df11=1-0.5*(1-alpha)*h*h*(-k-3*gama*y1a*y1a);
      df12=0.5*(1-alpha)*h*h*c;
      df21=-(1-beta)*h*(-k-3*gama*y1a*y1a);
      df22=1+(1-beta)*h*c;
%
      det=df11*df22-df12*df21;
      j11=df22/det;
      j12=-df12/det;
      j21=-df21/det;
      j22=df11/det;
%
      a2old=-c*y2old-k*y1old-gama*y1old*y1old*y1old+P*cos(2*pi*tm(i-1));
      a2new=-c*y2a-k*y1a-gama*y1a*y1a*y1a+P*cos(2*pi*tm(i));
%
      fun1=y1a-y1old-h*y2old-0.5*alpha*h*h*a2old;
      fun1=fun1-0.5*(1-alpha)*h*h*a2new -sigma*I10;
%
      fun2=y2a-y2old-beta*h*a2old;
      fun2=fun2-(1-beta)*h*a2new-sigma*db;
%
      dy1=j11*fun1+j12*fun2;
      dy2=j21*fun1+j22*fun2;
%
      y1new=y1a-dy1;
      y2new=y2a-dy2;
%
      y1a_2=y1new;
      y2a_2=y2new;
%
      err=(y1a_2-y1a_1)^2+(y2a_2-y2a_1)^2;
      err=sqrt(err);
%
      y1a=y1new;
      y2a=y2new;
%
    end
%
    dy(i,j)=y2new;
    y(i,j)=y1new;

    y2old=y2new;
    y1old=y1new;
%
  end
%
end

for i=1:N1
  meany(i)=mean(y(i,:));
  meandy(i)=mean(dy(i,:));
end
%
figure(1)
if (k==1)plot(tm,meany,'black'),hold on
end
%
figure(2)
if( k==1) plot(tm,meandy,'-k'),hold on
end
%
figure(1)
if (k==2)plot(tm,meany,'--black'),hold on
end
%
figure(2)
if( k==2) plot(tm,meandy,'--black'),hold on
end
%
figure(1)
if (k==3)plot(tm,meany,'.black','markersize',2),hold on
end
%
figure(2)
if( k==3) plot(tm,meandy,'.black','markersize',2),hold on
end
%
figure(1)
if (k==4)plot(tm,meany,'-.black','markersize',2),hold on
end
%
figure(2)
if( k==4) plot(tm,meandy,'-.black','markersize',2),hold on
end
%
end
%-------------------------------------------------------------------

