%                               Example5_5_Fig5_9.m 
%
%                           Stochastic Newmark method  
%                           (Equations 5.116 & 120)
%               
%      solution for Duffing oscillator (same as of Example 5.2) (see Equation 5.40)
%
%
%         Figure 5.9a contains:  1) solution for E[x(t)] by stochastic
%                                         Newmark method (Equation 5.123a)
%         Figure 5.9b contains:  2) solution for E[dx(t)] by stochastic
%                                         Newmark method (Equation 5.123b)
%   
%        Both the above Figures contain the reference solution obtained by 
%             Explicit EM method with dt=0.0001 
%                     (supposed to be the true solution)
%__________________________________________________________________________
%
clc
clear all
close all
%
pi=3.14159265;
T=5;
%
%
%parameters of the Duffing oscillator (Equation 5.40)
  c=4.0;
  k=100.0;
  gama=100.0;
  omega=2*pi; % frequency of harmonic excitation
  P=4.0;   % amplitude of the harmonic excitation
  sigma=0.01;   % noise intensity 
%
% Stochastic Newmark implicit parameters  (see Equations 5.116 and 120)
alpha=0.5;
beta=0.5;
%
%
N=1000;  % no. of samples in the MC run
%
%__________________________________________________________________________
%               Solution by Equation (5.123) - Stochastic Newmark method
%
'solution by Stochastic Newmark method with dt=0.05'
%
dt1=0.05;   % time step
h=dt1;      
tm=0:dt1:T;    % time variable
N1=numel(tm)
%
%
%
% initial conditions
x0=0.0;
dx0=0.0;
%
%
for j=1:N
    j   % sample counter
%
  y1old=x0;
  y2old=dx0;
  y(1)=y1old;
  dy(1)=y2old;
%
%  start of integration
  for i=2:N1
%
  u1=randn;
  u2=randn;
  u3=randn;
%
  db1=sqrt(dt1)*u2;
  db2=sqrt(dt1)*u3;
%
  db=sqrt(dt1)*u1;
%
  I10=0.5*dt1*sqrt(dt1)*(db1+db2/sqrt(3));
  I01=0.5*dt1*sqrt(dt1)*(db1-db2/sqrt(3));
%
    y1a=y1old;
    y2a=y2old;
%
    err=1;
%
    while err>1e-5
      y1a_1=y1a;
      y2a_1=y2a;
%
      df11=1-0.5*(1-alpha)*h*h*(-k-3*gama*y1a*y1a);
      df12=0.5*(1-alpha)*h*h*c;
      df21=-(1-beta)*h*(-k-3*gama*y1a*y1a);
      df22=1+(1-beta)*h*c;
%
      det=df11*df22-df12*df21;
      j11=df22/det;
      j12=-df12/det;
      j21=-df21/det;
      j22=df11/det;
%
      a2old=-c*y2old-k*y1old-gama*y1old*y1old*y1old+P*cos(2*pi*tm(i-1));
      a2new=-c*y2a-k*y1a-gama*y1a*y1a*y1a+P*cos(2*pi*tm(i));
%
      fun1=y1a-y1old-h*y2old-0.5*alpha*h*h*a2old;
      fun1=fun1-0.5*(1-alpha)*h*h*a2new -sigma*I10;
%
      fun2=y2a-y2old-beta*h*a2old;
      fun2=fun2-(1-beta)*h*a2new-sigma*db;
%
      dy1=j11*fun1+j12*fun2;
      dy2=j21*fun1+j22*fun2;
%
      y1new=y1a-dy1;
      y2new=y2a-dy2;
%
      y1a_2=y1new;
      y2a_2=y2new;
%
      err=(y1a_2-y1a_1)^2+(y2a_2-y2a_1)^2;
      err=sqrt(err);
%
      y1a=y1new;
      y2a=y2new;
%
    end
%
    dy(i,j)=y2new;
    y(i,j)=y1new;
%
    y2old=y2new;
    y1old=y1new;
%
  end

end
%
for i=1:N1
  meany(i)=mean(y(i,:));
  meandy(i)=mean(dy(i,:));
end
%
figure(1)      % Figure 5.9a
plot(tm,meany,'--black'),hold on         % plotting E[x(t)]
figure(2)      % Figure 5.9b
plot(tm,meandy,'--black'),hold on        % plotting E[dx(t)]
%
%
%
%
%
%__________________________________________________________________________
%                 reference solution by  Explicit EM method with dt=0.0001 
%                           (suposed to be the true solution)
%
'reference solution by EM method with dt=0.0001'
%
%initial conditions
x0=0.0;
dx0=0.0;
%
%
%     
dt=0.0001;
t=0:dt:T;
%
%
%
%
for j=1:N
    j  % sample run
  x1old=x0;
  x2old=dx0;
  x(1)=x1old;
  dx(1)=x2old;
%
% numerical solution by Explicit EM method for Duffing oscillator
%                       (Equation 5.36)
  for i=1:numel(t)-1
    force=P*cos(omega*t(i));
    dB=sqrt(dt)*randn;
    x1new=x1old+x2old*dt;
    x2new=x2old+(-c*x2old-k*x1old-gama*x1old^3+force)*dt+sigma*dB;
    dx(i+1,j)=x2new;
    x(i+1,j)=x1new;
    x1old=x1new;
    x2old=x2new;
  end
%
end
%
%
for i=1:numel(t)
  meanx(i)=mean(x(i,:));
  meandx(i)=mean(dx(i,:));
end
%plotting 
figure(1)    % Figure 5.9a
plot(t,meanx,'black'), hold on   % E[x(t)]
figure(2)    % Figure 5.9b
plot(t,meandx,'black'), hold on   % E[dx(t)]


