%                               Example5_9_Fig5_14.m  
%
%                Solution by Weak Stochastic Newmark method (WSNM)
%                   to the Duffing oscillator of Example 5.2
%
%
%                 moment time histories of displacement and velocity 
%                 with MSIs    -       Equations (5.123)a,b
%                 & equivalent RVs   - Equations (5.166)a,b 
%
%       Figure 5.14a: mean displacement time histories by WSNM using MSIs 
%                           and equivalent RVs   
%       Figure 5.14b: mean velocity time histories by WSNM using MSIs 
%                                   and equivalent RVs   
%__________________________________________________________________________
%
%
%
clc
clear all
close all
%
pi=3.14159265;
%
% 
T=5;    % total time of integration
%
% parameters pf the Duffing oscillator (Equation 5.40) of Example 5.2
 C=4.0;
 K=100.0;
 gama=100.0;
 sigma=0.01;  % noise intensity
 omega=2*pi;  % excitation frequency 
 P=4.0;       % amplitude of excitation
%
% Newmark method - implicit parameters
  alpha=0.5;
  beta=0.5;
%
%
N=100;   % no. of samples in MC run
%
% initial conditions
x0=0.0;
dx0=0.0;
%
%
%
for j=1:N
%
j   % sample counter
%
xwMSI(1,j)=x0;    % varaibels defined for method with MSIs
dxwMSI(1,j)=dx0;
%
% 
xwRVs(1,j)=x0;   % varaibels defined for method with RVS
dxwRVs(1,j)=dx0;
%
%__________________________________________________________________________
%        Solution by Equation (5.123) - Stochastic Newmark method
%                         with MSIs
%
%
%
dt1=2^-8;
h=dt1;
tm=0:dt1:T;
N1=numel(tm);
%
%
  y1old=x0;
  y2old=dx0;
  y(1)=y1old;
  dy(1)=y2old;
  z1old=x0;
  z2old=dx0;
  z(1)=z1old;
  dz(1)=z2old;
%
%
%
%
  for k=2:N1
%
%
%%--------------------------------------------------------------------------
%     start of 1st Run for stochastic Newmark method with MSIs - Equations (123)a,b
  u1=randn;
  u2=randn;
  u3=randn;
%
  db1=sqrt(dt1)*u2;
  db2=sqrt(dt1)*u3;
%
  db=sqrt(dt1)*u1;
%
  I10=0.5*dt1*(db1+db2/sqrt(3));
  I01=0.5*dt1*(db1-db2/sqrt(3));
%
    y1a=y1old;
    y2a=y2old;
%
    err=1;
    while err>1e-5
      y1a_1=y1a;
      y2a_1=y2a;
%
      df11=1-0.5*(1-alpha)*h*h*(-K-3*gama*y1a*y1a);
      df12=-0.5*(1-alpha)*h*h*(-C);
      df21=-(1-beta)*h*(-K-3*gama*y1a*y1a);
      df22=1-(1-beta)*h*(-C);
%
      det=df11*df22-df12*df21;
      j11=df22/det;
      j12=-df12/det;
      j21=-df21/det;
      j22=df11/det;
%
      a2old=-C*y2old-K*y1old-gama*y1old*y1old*y1old+P*cos(2*pi*tm(k-1));
      a2new=-C*y2a-K*y1a-gama*y1a*y1a*y1a+P*cos(2*pi*tm(k));
%
      fun1=y1a-y1old-h*y2old-0.5*alpha*h*h*a2old;
      fun1=fun1-0.5*(1-alpha)*h*h*a2new -sigma*I10;
%
      fun2=y2a-y2old-beta*h*a2old;
      fun2=fun2-(1-beta)*h*a2new-sigma*db;
%
      dy1=j11*fun1+j12*fun2;
      dy2=j21*fun1+j22*fun2;
%
      y1new=y1a-dy1;
      y2new=y2a-dy2;
%
      y1a_2=y1new;
      y2a_2=y2new;
%
      err=(y1a_2-y1a_1)^2+(y2a_2-y2a_1)^2;
      err=sqrt(err);
%
      y1a=y1new;
      y2a=y2new;
%
    end
%
    y2old=y2new;
    y1old=y1new;
%
      xwMSI(k,j)=y1new;    
      dxwMSI(k,j)=y2new;
%  end of 1st run for stochastic Newmark method with MSIs
%
%
%
%--------------------------------------------------------------------------
%  start of 2nd run for stochastic Newmark method with Equivalent RVs
%                               (Equation 5.166)
%
%    generation of discrete RVs 
          s1=(1.0/6.0);
          s2=s1+(2.0/3.0);
          s3=1.0;
          u4=rand;
          
% discrete RV in Equation 5.155a
          if(u4<=s1)
              drv1=-sqrt(3.0);   
          end
          if((u4>s1)&&(u4<=s2))
              drv1=0.0;
          end
          if((u4>s2)&&(u4<=s3))
              drv1=sqrt(3.0);
          end
%
          drv2=drv1/2.0;        % discrete RV in Equation 5.156a 
%  
          sqdt1=sqrt(dt1);
          Ir=drv1*sqdt1;            
          Jr=drv2*sqdt1*dt1;  
          %
%discrete RV in Equation 5.155b
          if(u4<=0.5)
              drv=-1.0;
          end
          if(u4>0.5)
              drv=1.0;
          end
%
          Iir=0.5*(drv1*drv1-drv*drv);   % RV in Equation 5.156b
%
    z1a=z1old;
    z2a=z2old;
%
    err=1;
%
    while err>1e-5
     z1a_1=z1a;
     z2a_1=z2a;
%
      df11=1-0.5*(1-alpha)*h*h*(-K-3*gama*z1a*z1a);
      df12=-0.5*(1-alpha)*h*h*(-C);
      df21=-(1-beta)*h*(-K-3*gama*z1a*z1a);
      df22=1-(1-beta)*h*(-C);
%
      det=df11*df22-df12*df21;
      j11=df22/det;
      j12=-df12/det;
      j21=-df21/det;
      j22=df11/det;
%
      a2old=-C*z2old-K*z1old-gama*z1old*z1old*z1old+P*cos(2*pi*tm(k-1));
      a2new=-C*z2a-K*z1a-gama*z1a*z1a*z1a+P*cos(2*pi*tm(k));
%
      fun1=z1a-z1old-h*z2old-0.5*alpha*h*h*a2old;
      fun1=fun1-0.5*(1-alpha)*h*h*a2new -sigma*Jr;
%
      fun2=z2a-z2old-beta*h*a2old;
      fun2=fun2-(1-beta)*h*a2new-sigma*Ir;
%
      dz1=j11*fun1+j12*fun2;
      dz2=j21*fun1+j22*fun2;
%
      z1new=z1a-dz1;
      z2new=z2a-dz2;
%
      z1a_2=z1new;
      z2a_2=z2new;
%
      err=(z1a_2-z1a_1)^2+(z2a_2-z2a_1)^2;
      err=sqrt(err);
%
      z1a=z1new;
      z2a=z2new;
%
    end
%
    z2old=z2new;
    z1old=z1new;
%
      xwRVs(k,j)=z1new;
      dxwRVs(k,j)=z2new;
% end of 2nd run for stochastic Newmark method with Equivalent RVs
%
%
%
%
%
  end    % end of integration     k=1:N1
%
end    % of N samples
%---------------------------------------------------------------
%
%
for k=1:N1
    xwM(k)=0.0;
    dxwM(k)=0.0;
    xwR(k)=0.0;
    dxwR(k)=0.0;
%    
    for j=1:N
%
% moment time histories with MSIs 
xwM(k)= xwM(k)+xwMSI(k,j)/N;
dxwM(k)= dxwM(k)+dxwMSI(k,j)/N;
%
%
% moment time histories with equivalent RVs 
xwR(k)= xwR(k)+xwRVs(k,j)/N;
dxwR(k)= dxwR(k)+ dxwRVs(k,j)/N;
%
    end
end
%
% plotting
figure(1)       % Figure 5.14a  -  mean displacement time histories 
plot(tm,xwM,'*black','markersize',2),hold on    % with MSIs
plot(tm,xwR,'black')               % with RVs
%
%
figure(2)       % Figure 5.14b  -  mean velocity time histories 
plot(tm,dxwM,'*black','markersize',2),hold on   % with MSIs
plot(tm,dxwR,'black')              % with RVs


