%                           Example6_1_Fig6_2.m
%                    
%		         Non-linear filtering by traditional EKF 
%                              (Section 6.3.1)
%                       
%     Non-linear  Duffing oscillator (Ghosh et al. - 
%               Comput. Methods Appl. Mech. Engrg. 196, 2007)
%
%           Figure(6.2a)  with  observation of displacement (H=[1 0])
%           Figure(6.2b)  with  observation of velocity (H=[0 1])
%
%---------------------------------------------------------------------
clc
clear all
close all
%
%
% data of Duffing oscillator (Equation 6.56)
c=0.05;
k=4;
alpha=7;
eta=c;  
w=sqrt(k);     % system frequency in rad/sec.
%
Lamda=1.5*w;   % excitation frequency in rad/sec.
A=3;           % amplitude of the excitation (harmonic) force
%
%
sigx=1;      % process noise intensity  (Equation (6.56)
Q=[0 0.01;0.01 1.0];   % initial process noise covariance matrix;
%
%
sigy=0.03;    % measurement noise intensity
R = 0.009;    % measurement noise variance (in general a matrix)
%
% 
%----------------------------------------------------------------------
% initial conditions
x0=0;
dx0=0;
%
T=30;   % total time of integraion in sec.
dt=0.01;   % time step
t=0:dt:T;   % time variable
M=numel(t);
h=dt;
%
%
%___________________________________________________________________________
%                   CASE 1  filtering with displacement measurement
H = [1 0]; % measurement matrix for x1 - displacement ( for Figure 6.2a)
%              --------------------------------------------------
%
%
Qpre=Q;   %initialization of process noise covaraince matrix
% set up the size of the innovations vector
Inn = zeros(size(R));
%
N=1;      % no. simulations
%
for j=1:N
  j             % simulation counter
x = [0 0]; % initial state vector
P = [0.01 0;0 0.01]; % initial predicted error covariance matrix 
%
xUp=x;
xTr=x;
PUp=P;
%
% Initialization before start of integration
for i=1:M
    xest(i,j)=0.0;
    dxest(i,j)=0.0;
    xestr(i,j)=0.0;
    dxestr(i,j)=0.0;
end
%
%
% start of integraion
for i=1:M
   t(i) ;
%
% prediction  -  solution without noise
  ft=A*sin(Lamda*t(i));   % external force - harmonic 
%
 xx2=xUp(2) + (-2*eta*w*xUp(2) - w^2*xUp(1) - alpha*xUp(1)^3)*dt + ft*dt;
%
 xx1=xUp(1) + xx2*dt;
 xpre = [xx1 xx2];
%
%
%
% find true solution for simulation of measurement
xx2=xTr(2) + (-2*eta*w*xTr(2) - w^2*xTr(1) - alpha*xTr(1)^3)*dt + ft*dt ;
xx1=xTr(1) + xx2*dt;
xtrue= [xx1 xx2];
%
%
%    
%  
% To compute Qpre - process error covariance Equation 6.48
tmin=t(k-1);
tmax=t(k);
%
y1a=xUp(1);
y2a=xUp(2);
% %
% phi matrix -- Equation 6.47
A1=k+3*alpha*y1a^2;
phi11=1-0.5*h^2*A1+(1/6)*h^3*c*A1;
phi12=h-0.5*h^2*c+(1/6)*h^3*(c^2-A1);
phi21=-h*A1+0.5*h^2*c*A1+(1/6)*h^3*(A1^2-c^2*A1);
phi22=1-h*c+0.5*h^2*(c^2-A1)+(1/6)*h^3*(2*c*A1-c^3);    
      phi=zeros(2,2);
%        
      phi(1,1)=phi11;
      phi(1,2)=phi12;
      phi(2,1)=phi21;
      phi(2,2)=phi22; 
%
 Phat = phi*PUp*phi' + Qpre;   % Equation 6.46
 %
 %
 % Simulate the measurement
 u1=sqrt(dt*sigy)*randn;          % Brownian motion in measurement equation 6.44
 z = H * xtrue' + u1;    % H=[1 0] for Figure 6.2a and H=[0 1] for  Figure 6.2b 
 Yobs(i,j)=z;  
%
%
%
% Update system states  (follow steps 3 to 4)
    % Innovation
    Inn = Yobs(i,j) - H * xpre';
%  
    % Covariance of Innovation  
    s = H * Phat * H' + R;
%
    % Gain matrix   Equation 6.49
    K = Phat * H' * inv(s);
%
    % update State estimate
    xUp = xpre + K' * Inn;    % Equation 6.50a
    xTr=xtrue;
%
    % Covariance of prediction error
    PUp = Phat - K*H*Phat;   % Equation 6.50b)
%   
    xest(i,j)=xUp(1);     % estimate of displacement
    dxest(i,j)=xUp(2);    % estimate of velocity
%    
 xestr(i,j)=xTr(1);     % true solution for displacement
 dxestr(i,j)=xTr(2);    % true solution for velocity
 %   
  %
  % find Q(j+1) in Equation 6.48
 hj=tmax-tmin;     
Iphi12j=hj^2/2 - c*hj^3/6 + (c^2-A1)*hj^4/24;
%
hk=tmax-tmin;     
Iphi12k=hk^2/2 - c*hk^3/6 + (c^2-A1)*hk^4/24;
%
Iphi22k=hk-c*hk^2/2 + hk^3/6*(c^2-A1) + hk^4/24*(2*c*A1-c^3);
%
Iphi22j=hj-c*hj^2/2 + hj^3/6*(c^2-A1) + hj^4/24*(2*c*A1-c^3);
Qpre(1,1)=sigx^2*Iphi12j*Iphi12k;
Qpre(1,2)=sigx^2*Iphi12j*Iphi22k;
Qpre(2,1)=Qpre(1,2);
Qpre(2,2)=sigx^2*Iphi22j*Iphi22k;
%
end
%
%
end
%
%
%
% Average for observed solution over N simulations
for k=1:M    
    obs(k)=0.0;  
  for j=1:N
    obs(k)= obs(k)+Yobs(k,j)/N;  
  end
end
%
%
%
%                       Plotting results for Case 1
%
%
%  Find averages over the N simlations and plot Figure 6.2a
% 
for k=1:M
 %   
    xwM(k)=0.0;  
%
    for j=1:N
    xwM(k)= xwM(k)+xest(k,j)/N;   % estimate of displacement
    end
%    
end
%
% 
figure(1)           %   Figure 6.2a
plot(t,obs,'--black','markersize',5), hold on   %  observed solution of displacement 
%                     
plot(t,xwM,'black','markersize',2)          %  estimated solution of displacement 
%
%__________________________________________________________________________

%               CASE 2  filtering with velocity measurement
H = [0 1]; % measurement matrix for x2 - velocity   ( for Figure 6.2b)
%               --------------------------------------------------
%
%
Qpre=Q;   %initialization of process noise covaraince matrix
% set up the size of the innovations vector
Inn = zeros(size(R));
%
N=1;      % no. simulations
%
for j=1:N
  j             % simulation counter
x = [0 0]; % initial state vector
P = [0.01 0;0 0.01]; % initial predicted error covariance matrix 
%
xUp=x;
xTr=x;
PUp=P;
%
% Initialization before start of integration
for i=1:M
    xest(i,j)=0.0;
    dxest(i,j)=0.0;
    xestr(i,j)=0.0;
    dxestr(i,j)=0.0;
end
%
%
% start of integraion
for i=1:M
   t(i) ;
%
% prediction  -  solution without noise
  ft=A*sin(Lamda*t(i));   % external force - harmonic 
%
 xx2=xUp(2) + (-2*eta*w*xUp(2) - w^2*xUp(1) - alpha*xUp(1)^3)*dt + ft*dt;
%
 xx1=xUp(1) + xx2*dt;
 xpre = [xx1 xx2];
%
%
%
% find true solution for simulation of measurement
xx2=xTr(2) + (-2*eta*w*xTr(2) - w^2*xTr(1) - alpha*xTr(1)^3)*dt + ft*dt ;
xx1=xTr(1) + xx2*dt;
xtrue= [xx1 xx2];
%
%
%    
%  
% To compute Qpre - process error covariance Equation 6.48
tmin=t(k-1);
tmax=t(k);
%
y1a=xUp(1);
y2a=xUp(2);
% %
% phi matrix -- Equation 6.47
A1=k+3*alpha*y1a^2;
phi11=1-0.5*h^2*A1+(1/6)*h^3*c*A1;
phi12=h-0.5*h^2*c+(1/6)*h^3*(c^2-A1);
phi21=-h*A1+0.5*h^2*c*A1+(1/6)*h^3*(A1^2-c^2*A1);
phi22=1-h*c+0.5*h^2*(c^2-A1)+(1/6)*h^3*(2*c*A1-c^3);    
      phi=zeros(2,2);
%        
      phi(1,1)=phi11;
      phi(1,2)=phi12;
      phi(2,1)=phi21;
      phi(2,2)=phi22; 
%
 Phat = phi*PUp*phi' + Qpre;   % Equation 6.46
 %
 %
 % Simulate the measurement
 u1=sqrt(dt*sigy)*randn;          % Brownian motion in measurement equation 6.44
 z = H * xtrue' + u1;    
 Yobs(i,j)=z;  
%
%
%
% Update system states  (follow steps 3 to 4)
    % Innovation
    Inn = Yobs(i,j) - H * xpre';
%  
    % Covariance of Innovation  
    s = H * Phat * H' + R;
%
    % Gain matrix   Equation 6.49
    K = Phat * H' * inv(s);
%
    % update State estimate
    xUp = xpre + K' * Inn;    % Equation 6.50a
    xTr=xtrue;
%
    % Covariance of prediction error
    PUp = Phat - K*H*Phat;   % Equation 6.50b)
%   
    xest(i,j)=xUp(1);     % estimate of displacement
    dxest(i,j)=xUp(2);    % estimate of velocity
%    
 xestr(i,j)=xTr(1);     % true solution for displacement
 dxestr(i,j)=xTr(2);    % true solution for velocity
 %   
  %
  % find Q(j+1) in Equation 6.48
 hj=tmax-tmin;     
Iphi12j=hj^2/2 - c*hj^3/6 + (c^2-A1)*hj^4/24;
%
hk=tmax-tmin;     
Iphi12k=hk^2/2 - c*hk^3/6 + (c^2-A1)*hk^4/24;
%
Iphi22k=hk-c*hk^2/2 + hk^3/6*(c^2-A1) + hk^4/24*(2*c*A1-c^3);
%
Iphi22j=hj-c*hj^2/2 + hj^3/6*(c^2-A1) + hj^4/24*(2*c*A1-c^3);
Qpre(1,1)=sigx^2*Iphi12j*Iphi12k;
Qpre(1,2)=sigx^2*Iphi12j*Iphi22k;
Qpre(2,1)=Qpre(1,2);
Qpre(2,2)=sigx^2*Iphi22j*Iphi22k;
%
end
%
%
end
%
%
%
% Average for observed solution over N simulations
for k=1:M    
    obs(k)=0.0;  
  for j=1:N
    obs(k)= obs(k)+Yobs(k,j)/N;  
  end
end
%
%
%                       Plotting results for Case 2
%
%
%  Find averages over the N simlations and plot Figure 6.2b
% 
for k=1:M
 %     
    dxwM(k)=0.0;
%
    for j=1:N  
      dxwM(k)= dxwM(k)+dxest(k,j)/N;
    end
%    
end
%
% 
figure(2)           % Figure 6.2b
plot(t,obs,'--black'), hold on     % observed solution of velocity
%                    
plot(t,dxwM,'black')            % estimated solution of velocity 
%


