%                       Fig6_12.m
%
%          Parameter estimation by EnKF (Section 6.4.3)  Vs.  ABS filter
%                       
%                  5 floor shear building
%
%                The shear frame is modelled as a stick model 
%
%           This program contains two runs:
%                       1. result by EnKF - Figures 6.12a and 6.12b
%                       2. result by ABS  - Figures 6.12c and 6.12d
%                                            
%
%-----------------------------------------------------------------------
clc
clear all
close all
%
n=5;                     % no. of floors
L=144;                   % height of each floor in inches
L=L*0.0254;              % heght in meters
%
m=12;                    % mass of each floor in kipssec^2/in                                       
m=m*175100   ;            % mass = 2101200  kgs
%
E=29000;                 % in kips/sqin
E=E*1000/0.00014503773;   % in N/m^2
%
I=3200;                  % Inertia in in^4
I=I*0.0254^4;            % in m^4
%
k=4*(12*E*I/L^3);        % total stiffness of four columns (in parallel) between any two floors
%
% Mass matrix (diagonal)
M=[m 0 0 0 0 0;0 m 0 0 0 0;0 0 m 0 0 0;0 0 0 m 0 0;0 0 0 0 m 0;0 0 0 0 0 m];
%
M_original=M ;       %  from bottom to top floor
%
%
% stiffness matrix (symmetric)
K=[2*k -k 0 0 0 0;-k 2*k -k 0 0 0;0 -k 2*k -k 0 0;0 0 -k 2*k -k 0;0 0 0 -k 2*k -k;0 0 0 0 -k k];
%
K_original=K;
%
K_normalized=inv(M_original)*K_original ; % mass normalized stiffness matrix - inv(M)*K
%
%
%_________________________________________________________________________________
% to obtain (nprmalized) stiffnesses of columns inbetween floors for the stick model
K1=K_normalized(1,1)/2 ; % for the 1st column (bottom)
K2=K1 ;                  % for the 2nd column
K3=K1  ;                 % for the 3rd column
K4=K1  ;                 % for the 4th column
K5=K1  ;                 % for the 5th column (top)
%
%  to form the diagonal stiffness terms in the stiffness matrix
K11=K1+K2 ;    % at the 1st floor level  (diagonal term K(1,1))
K22=K2+K3 ;    % at the 2nd floor level  (diagonal term K(2,2))
K33=K3+K4 ;    % at the 3rd floor level  (diagonal term K(3,3))
K44=K4+K5  ;   % at the 4th floor level  (diagonal term K(4,4))
K55=K5    ;    % at the 5th floor level  (diagonal term K(5,5))
%
% note that the cross stiffness terms are negative
  K12=-K2;      % K(1,2)=-K2=K(2,1) 
  K21=-K2;
%  
  K23=-K3;      % K(2,3)=-K3=K(3,2)
  K32=-K3;
%
  K34=-K4;      % K(3,4)=-K4=K(4,3)
  K43=-K4; 
%
  K45=-K5;      % K(4,5)=-K5=K(5,4)
  K54=-K5;
%___________________________________________________________________________________
%
%
C1=5.225*10^6/ 2101200 ;  % damping of each column in between floors (Rayleigh proportional damping)
C2=C1;
C3=C1;
C4=C1;
C5=C1;
%  to form the diagonal damping terms in the stiffness matrix
C11=C1+C2 ;   % at the 1st floor level  (diagonal term C(1,1))
C22=C2+C3 ;   % at the 2nd floor level  (diagonal term C(2,2))
C33=C3+C4 ;   % at the 3rd floor level  (diagonal term C(3,3))
C44=C4+C5 ;   % at the 4th floor level  (diagonal term C(4,4))
C55=C5    ;   % at the 5th floor level  (diagonal term C(5,5))
%
% note that the cross damping terms are negative
  C12=-C2;      % C(1,2)=-C2=C(2,1) 
  C21=-C2;
%  
  C23=-C3;      % C(2,3)=-C3=C(3,2)
  C32=-C3;
%
  C34=-C4;      % C(3,4)=-C4=C(4,3)
  C43=-C4; 
%
  C45=-C5;      % C(4,5)=-C5=C(5,4)
  C54=-C5;
%-----------------------------------------------------------------------------
%               Eigenvalue analysis   (only to observe the natural 
%                                           frequencies of the system)
%
%    [V,Lamda]=eigs(K,M,n)   
%
%   Lamda =7.2258, 62.5369,  160.4881, 278.6398, 389.925, 468.8495    Rad/sec.
%------------------------------------------------------------------------------   
%
%
%   Harmonic suport excitation
%   Seismic acceleration time history - for example - a half sine wave 
pi=3.14159265;
Nodalforce=zeros(1,n);  % row vector
%
%
Ex_freq=1;  % in rad/sec.
%
% assuming a harmonic force of 0.1*sin(2*pi*Ex_freq*t)
% at the support
% 0.1 is the harmonic excitation amplitude
% 
Nodalforce=[0.1,0.1,0.1,0.1,0.1] ; % see Clough % Penzien 1993
%__________________________________________________________________________
%
%
%                 Parameter estimation starts from here
%
%          Total parameters = 5 stiffness + 5 damping parameters
%  Total state variables (along with 10 augmented variables) = 2*5 + 10 =20
%
%__________________________________________________________________________  
%
%
%                       start of computation by EnKF
%
T=20;
Np=500;   % no. of particles 
N=Np;
%
dt=0.01;
t=0:dt:T;
n_sub_in=10;
%
D=10;    % no. of system variables ( 5 displacements and 5 velocities)
%
Nm=5;    % no. of measurements ( = 5 displacements at the 5 floor levels) 
% 
aux_it=1;
Index=9;
%--------------------------------------------------------------------------
% generate reference (true) solution without noise
X_mess(:,1)=[0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0] ; % initialization 
%
t1=0:dt/n_sub_in:T;
   for ijk=1:numel(t1)-1
       ft=Nodalforce(1)*sin(2*pi*Ex_freq*t1(ijk+1));
    X_mess(1,ijk+1)=X_mess(1,ijk)+X_mess(2,ijk)*dt/n_sub_in;
    X_mess(2,ijk+1)=X_mess(2,ijk)+[-K11*X_mess(1,ijk)+K2*X_mess(3,ijk)]*dt/n_sub_in;
    X_mess(2,ijk+1)=X_mess(2,ijk+1)+[-C11*X_mess(2,ijk)+C2*X_mess(4,ijk)]*dt/n_sub_in;
    X_mess(2,ijk+1)=X_mess(2,ijk+1)+[ft]*dt/n_sub_in;
    
    X_mess(3,ijk+1)=X_mess(3,ijk)+X_mess(4,ijk)*dt/n_sub_in;
    X_mess(4,ijk+1)=X_mess(4,ijk)+[-K22*X_mess(3,ijk)+K3*X_mess(5,ijk)]*dt/n_sub_in;
    X_mess(4,ijk+1)=X_mess(4,ijk+1)+[-C22*X_mess(4,ijk)+C3*X_mess(6,ijk)]*dt/n_sub_in;
    X_mess(4,ijk+1)=X_mess(4,ijk+1)+[K2*X_mess(1,ijk)+C2*X_mess(2,ijk)]*dt/n_sub_in;
    X_mess(4,ijk+1)=X_mess(4,ijk+1)+ft*dt/n_sub_in;
    
    X_mess(5,ijk+1)=X_mess(5,ijk)+X_mess(6,ijk)*dt/n_sub_in;
    X_mess(6,ijk+1)=X_mess(6,ijk)+[-K33*X_mess(5,ijk)+K4*X_mess(7,ijk)]*dt/n_sub_in;
    X_mess(6,ijk+1)=X_mess(6,ijk+1)+[-C33*X_mess(6,ijk)+C4*X_mess(8,ijk)]*dt/n_sub_in;
    X_mess(6,ijk+1)=X_mess(6,ijk+1)+[K3*X_mess(3,ijk)+C3*X_mess(4,ijk)]*dt/n_sub_in;
    X_mess(6,ijk+1)=X_mess(6,ijk+1)+ft*dt/n_sub_in;
    
    X_mess(7,ijk+1)=X_mess(7,ijk)+X_mess(8,ijk)*dt/n_sub_in;
    X_mess(8,ijk+1)=X_mess(8,ijk)+[-K44*X_mess(7,ijk)+K5*X_mess(9,ijk)]*dt/n_sub_in;
    X_mess(8,ijk+1)=X_mess(8,ijk+1)+[-C44*X_mess(8,ijk)+C5*X_mess(10,ijk)]*dt/n_sub_in;
    X_mess(8,ijk+1)=X_mess(8,ijk+1)+[K4*X_mess(5,ijk)+C4*X_mess(6,ijk)]*dt/n_sub_in;
    X_mess(8,ijk+1)=X_mess(8,ijk+1)+ft*dt/n_sub_in;    
     
    X_mess(9,ijk+1)=X_mess(9,ijk)+X_mess(10,ijk)*dt/n_sub_in;
    X_mess(10,ijk+1)=X_mess(10,ijk)+[-K55*X_mess(9,ijk)+K5*X_mess(7,ijk)]*dt/n_sub_in;
    X_mess(10,ijk+1)=X_mess(10,ijk+1)+[-C55*X_mess(10,ijk)+C5*X_mess(8,ijk)]*dt/n_sub_in;
    X_mess(10,ijk+1)=X_mess(10,ijk+1)+ft*dt/n_sub_in;     % since K2 & C2 are oroginally -ve, 
 %   
   end
%
X_mes=X_mess(:,[1:n_sub_in:numel(t1)]);
X_ref(1:10,:)=X_mes;                    % referense (true) solution for states 
%
%--------------------------------------------------------------------------
%  generate measurements (= true solution + measurement noise)
for ijk=1:numel(t)
    X_mes(1:10,ijk)=X_mes(1:10,ijk)+0.05*max(X_mes(1:10,:)')'*randn(1);
end
%
z(1:Nm,:)=X_mes(1:2:Index,:); % Measurements on displacements of the 5 floors
%--------------------------------------------------------------------------
% generate initial state veraibles ( = 20 including stiffness and damping parameters) 
sigma_meas=0.1*diag(max(z'))/sqrt(dt);   %measurement noise
%
sig_pros=0.001; %%process noise
%
sigma_pross=zeros(2,2);
sigma_pross(2,2)=sig_pros;
%
sigma_K=0.5;        % intensity of BM for stiffness parameter  similar to  Equation 6.78c              
sigma_C=0.5;        % intensity of BM for damping parameter  similar to  Equation 6.78d     
%
Randn1=randn(2,Np);           
Randn2=randn(2,Np);
Randn3=randn(2,Np);
Randn4=randn(2,Np);
Randn5=randn(2,Np);
%
% randn6=rand(1,Np);
%---------------------------------------------------------------------
% Initial displacement and velocity ensemble
X(1:2,:)=[0*ones(1,Np);0*ones(1,Np)]+1e4*sigma_pross(2,2)*sqrt(dt)*Randn1; 
X(3:4,:)=[0*ones(1,Np);0*ones(1,Np)]+1e4*sigma_pross(2,2)*sqrt(dt)*Randn2;
X(5:6,:)=[0*ones(1,Np);0*ones(1,Np)]+1e4*sigma_pross(2,2)*sqrt(dt)*Randn3; 
X(7:8,:)=[0*ones(1,Np);0*ones(1,Np)]+1e4*sigma_pross(2,2)*sqrt(dt)*Randn4;
X(9:10,:)=[0*ones(1,Np);0*ones(1,Np)]+1e4*sigma_pross(2,2)*sqrt(dt)*Randn5; 
%
X(11,:)=0.4*K1+(1*K1-0.4*K1)*rand(1,Np);
X(12,:)=0.4*K2+(1*K2-0.4*K2)*rand(1,Np);
X(13,:)=0.4*K3+(1*K3-0.4*K3)*rand(1,Np);
X(14,:)=0.4*K4+(1*K4-0.4*K4)*rand(1,Np);
X(15,:)=0.4*K5+(1*K5-0.4*K5)*rand(1,Np);
%
X(16,:)=0.4*C1+(1*C1-0.4*C1)*rand(1,Np);
X(17,:)=0.4*C2+(1*C2-0.4*C2)*rand(1,Np);
X(18,:)=0.4*C3+(1*C3-0.4*C3)*rand(1,Np);
X(19,:)=0.4*C4+(1*C4-0.4*C4)*rand(1,Np);
X(20,:)=0.4*C5+(1*C5-0.4*C5)*rand(1,Np);
%
X_s(:,1)=mean(X,2);   % initial mean values for system, stiffness and damping parameters
                        %(for plotting purposes)
%   
tic
for i=1:numel(t)-1
    i    % time counter
%     
%           
      for j=1:Np
  %    prediction stage   
        ft=Nodalforce(1)*sin(2*pi*Ex_freq*t(i+1));
        X1(1,j)=X(1,j)+ X(2,j)*dt;
        X1(2,j)=X(2,j)-( (X(11,j)+X(12,j))*X(1,j)- X(12,j)*X(3,j) )*dt;
        X1(2,j)=X1(2,j)-( (X(16,j)+X(17,j))*X(2,j)-X(17,j)*X(4,j) )*dt;
        X1(2,j)=X1(2,j)+ ft*dt;
        X1(2,j)=X1(2,j) + sig_pros*sqrt(dt)*randn;    
    
        X1(3,j)=X(3,j)+ X(4,j)*dt;
        X1(4,j)=X(4,j)-( (X(12,j)+X(13,j))*X(3,j)- X(13,j)*X(5,j)- X(12,j)*X(1,j) )*dt;
        X1(4,j)=X1(4,j)-( (X(17,j)+X(18,j))*X(4,j)- X(18,j)*X(6,j)- X(17,j)*X(2,j))*dt;
        X1(4,j)=X1(4,j)+ ft*dt;
        X1(4,j)=X1(4,j) + sig_pros*sqrt(dt)*randn;
 %   
        X1(5,j)=X(5,j)+ X(6,j)*dt;
        X1(6,j)=X(6,j)-( (X(13,j)+X(14,j))*X(5,j)- X(14,j)*X(7,j)- X(13,j)*X(3,j) )*dt;
        X1(6,j)=X1(6,j)-( (X(18,j)+X(19,j))*X(6,j)- X(19,j)*X(8,j)- X(18,j)*X(4,j))*dt;
        X1(6,j)=X1(6,j)+ ft*dt;
        X1(6,j)=X1(6,j) + sig_pros*sqrt(dt)*randn(1,1);
 %   
        X1(7,j)=X(7,j)+ X(8,j)*dt;
        X1(8,j)=X(8,j)-( (X(14,j)+X(15,j))*X(7,j)- X(15,j)*X(9,j)- X(14,j)*X(5,j) )*dt;
        X1(8,j)=X1(8,j)-( (X(19,j)+X(20,j))*X(8,j)- X(20,j)*X(10,j)- X(19,j)*X(6,j))*dt;
        X1(8,j)=X1(8,j)+ ft*dt;
        X1(8,j)=X1(8,j) + sig_pros*sqrt(dt)*randn(1,1);
 %   
        X1(9,j)=X(9,j)+ X(10,j)*dt;
        X1(10,j)=X(10,j)-(X(15,j)*X(9,j)- X(15,j)*X(7,j) )*dt;
        X1(10,j)=X1(10,j)-(X(20,j)*X(10,j)- X(20,j)*X(8,j))*dt;
        X1(10,j)=X1(10,j)+ ft*dt;
        X1(10,j)=X1(10,j) + sig_pros*sqrt(dt)*randn(1,1);     
 %   
        X1(11,j)=X(11,j)+ sigma_K*sqrt(dt)*randn;
        X1(12,j)=X(12,j)+ sigma_K*sqrt(dt)*randn;
        X1(13,j)=X(13,j)+ sigma_K*sqrt(dt)*randn;
        X1(14,j)=X(14,j)+ sigma_K*sqrt(dt)*randn;        
        X1(15,j)=X(15,j)+ sigma_K*sqrt(dt)*randn;
                
        X1(16,j)=X(16,j)+ sigma_C*sqrt(dt)*randn;
        X1(17,j)=X(17,j)+ sigma_C*sqrt(dt)*randn;
        X1(18,j)=X(18,j)+ sigma_C*sqrt(dt)*randn;
        X1(19,j)=X(19,j)+ sigma_C*sqrt(dt)*randn;
        X1(20,j)=X(20,j)+ sigma_C*sqrt(dt)*randn;
 %     
      end      
%
%-----------------------------------------------------------------------
        mean_X1=mean(X1,2);
        for u=1:Np
           Xip(:,u)=(1/sqrt(Np-1))*(X1(:,u)-mean_X1); % predicton error anamoly matrix (Equation 6.91)
           Yi(:,u)=X1(1:2:Index,u);
        end
 %       
        mean_Yi=mean(Yi,2);
        for j=1:Np
            Yip(:,j)=(1/sqrt(Np-1))*(Yi(:,j)-mean_Yi);% measurement error anamoly matrix 
        end
 %
        Siinv=inv((Yip*Yip'+sigma_meas'*sigma_meas*dt));
        K_EnKF= Xip*Yip'*Siinv;     % Gain matrix - Equation 6.93
        mean(X1,2);
        z(:,i+1);
        X1(1:2:Index,:);
 %     
        for jj=1:2:Index
            jjj=(jj+1)/2;
        summ=0.0;
            for j=1:Np
            summ=summ+X1(jj,j)/Np;
            end
        meanX1(jjj)=summ;
        end
        meanX1';
        mean(X1(1:2:Index,2));
%            
       for u=1:Np
           for v=1:Np
               Ad(u,v)=0;
           end
           Ad(u,u)=1.0;
       end
 %      
        Cov_X=Xip*(Ad-Yip'*Siinv*Yip)*Xip'; % error covariance - Equation 6.95
 %
       for u=1:Np 
         X2(:,u)=X1(:,u) + K_EnKF*(z(:,i+1)-X1(1:2:Index,u)); % update - Equation 6.94
       end
      X2_mean=mean(X2,2);
%
      X=X2;  
      X_s(:,i+1)=X2_mean ;
% 
end
%
toc   
%        
        figure(1),hold on       % Figure 6.12a
        plot(t,2101200*K1*ones(1,size(t,2)),'black'); hold on 
        plot(t,2101200*X_s(11,:),'black'); hold on        
        plot(t,2101200*X_s(12,:),'black'); hold on        
        plot(t,2101200*X_s(13,:),'black'); hold on         
        plot(t,2101200*X_s(14,:),'black'); hold on         
        plot(t,2101200*X_s(15,:),'black'); hold on 
%        
        figure(2),hold on      % Figure 6.12b
        plot(t,2101200*C1*ones(1,size(t,2)),'black'); hold on 
        plot(t,2101200*X_s(16,:),'black'); hold on        
        plot(t,2101200*X_s(17,:),'black'); hold on 
        plot(t,2101200*X_s(18,:),'black'); hold on         
        plot(t,2101200*X_s(19,:),'black'); hold on 
        plot(t,2101200*X_s(20,:),'black'); hold on 
% 
%
%
%
%
%
%__________________________________________________________________________
% 
%               Parameter estimation by ABS filter
%
T=20;
Np=500;
N=Np;
%
dt=0.01;
t=0:dt:T;
n_sub_in=10;
%
D=10;
Nm=5;
aux_it=1;
Index=9;
%
X_mess(:,1)=[0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0]  % column vector
%%% Generating measurements
%
%
% reference solution
t1=0:dt/n_sub_in:T;
   for ijk=1:numel(t1)-1
       ft=Nodalforce(1)*sin(2*pi*Ex_freq*t1(ijk+1));
    X_mess(1,ijk+1)=X_mess(1,ijk)+X_mess(2,ijk)*dt/n_sub_in;
    X_mess(2,ijk+1)=X_mess(2,ijk)+[-K11*X_mess(1,ijk)+K2*X_mess(3,ijk)]*dt/n_sub_in;
    X_mess(2,ijk+1)=X_mess(2,ijk+1)+[-C11*X_mess(2,ijk)+C2*X_mess(4,ijk)]*dt/n_sub_in;
    X_mess(2,ijk+1)=X_mess(2,ijk+1)+[ft]*dt/n_sub_in;
    
    X_mess(3,ijk+1)=X_mess(3,ijk)+X_mess(4,ijk)*dt/n_sub_in;
    X_mess(4,ijk+1)=X_mess(4,ijk)+[-K22*X_mess(3,ijk)+K3*X_mess(5,ijk)]*dt/n_sub_in;
    X_mess(4,ijk+1)=X_mess(4,ijk+1)+[-C22*X_mess(4,ijk)+C3*X_mess(6,ijk)]*dt/n_sub_in;
    X_mess(4,ijk+1)=X_mess(4,ijk+1)+[K2*X_mess(1,ijk)+C2*X_mess(2,ijk)]*dt/n_sub_in;
    X_mess(4,ijk+1)=X_mess(4,ijk+1)+ft*dt/n_sub_in;
    
    X_mess(5,ijk+1)=X_mess(5,ijk)+X_mess(6,ijk)*dt/n_sub_in;
    X_mess(6,ijk+1)=X_mess(6,ijk)+[-K33*X_mess(5,ijk)+K4*X_mess(7,ijk)]*dt/n_sub_in;
    X_mess(6,ijk+1)=X_mess(6,ijk+1)+[-C33*X_mess(6,ijk)+C4*X_mess(8,ijk)]*dt/n_sub_in;
    X_mess(6,ijk+1)=X_mess(6,ijk+1)+[K3*X_mess(3,ijk)+C3*X_mess(4,ijk)]*dt/n_sub_in;
    X_mess(6,ijk+1)=X_mess(6,ijk+1)+ft*dt/n_sub_in;
    
    X_mess(7,ijk+1)=X_mess(7,ijk)+X_mess(8,ijk)*dt/n_sub_in;
    X_mess(8,ijk+1)=X_mess(8,ijk)+[-K44*X_mess(7,ijk)+K5*X_mess(9,ijk)]*dt/n_sub_in;
    X_mess(8,ijk+1)=X_mess(8,ijk+1)+[-C44*X_mess(8,ijk)+C5*X_mess(10,ijk)]*dt/n_sub_in;
    X_mess(8,ijk+1)=X_mess(8,ijk+1)+[K4*X_mess(5,ijk)+C4*X_mess(6,ijk)]*dt/n_sub_in;
    X_mess(8,ijk+1)=X_mess(8,ijk+1)+ft*dt/n_sub_in;    
     
    X_mess(9,ijk+1)=X_mess(9,ijk)+X_mess(10,ijk)*dt/n_sub_in;
    X_mess(10,ijk+1)=X_mess(10,ijk)+[-K55*X_mess(9,ijk)+K5*X_mess(7,ijk)]*dt/n_sub_in;
    X_mess(10,ijk+1)=X_mess(10,ijk+1)+[-C55*X_mess(10,ijk)+C5*X_mess(8,ijk)]*dt/n_sub_in;
    X_mess(10,ijk+1)=X_mess(10,ijk+1)+ft*dt/n_sub_in;     % since K2 & C2 are oroginally -ve, 
    
   end

X_mes=X_mess(:,[1:n_sub_in:numel(t1)]);
X_ref(1:10,:)=X_mes;                    % referense solution for states
%
%
%
%---------------------------------------------------------------------

for ijk=1:numel(t)
    X_mes(1:10,ijk)=X_mes(1:10,ijk)+0.05*max(X_mes(1:10,:)')'*randn(1);
end

z(1:Nm,:)=X_mes(1:2:Index,:); %%% Measurements on displacements of the 5 floors


sigma_meas=0.1*diag(max(z'))/sqrt(dt);   %measurement noise

sig_pros=0.001; %%process noise

sigma_pross=zeros(2,2);
sigma_pross(2,2)=sig_pros;

sigma_K=0.5;                
sigma_C=0.5;      

Randn1=randn(2,Np);           
Randn2=randn(2,Np);
Randn3=randn(2,Np);
Randn4=randn(2,Np);
Randn5=randn(2,Np);
%
%---------------------------------------------------------------------
% Initial displacement and velocity ensemble
X(1:2,:)=[0*ones(1,Np);0*ones(1,Np)]+1e4*sigma_pross(2,2)*sqrt(dt)*Randn1; 
X(3:4,:)=[0*ones(1,Np);0*ones(1,Np)]+1e4*sigma_pross(2,2)*sqrt(dt)*Randn2;
X(5:6,:)=[0*ones(1,Np);0*ones(1,Np)]+1e4*sigma_pross(2,2)*sqrt(dt)*Randn3; 
X(7:8,:)=[0*ones(1,Np);0*ones(1,Np)]+1e4*sigma_pross(2,2)*sqrt(dt)*Randn4;
X(9:10,:)=[0*ones(1,Np);0*ones(1,Np)]+1e4*sigma_pross(2,2)*sqrt(dt)*Randn5; 

X(11,:)=0.4*K1+(1*K1-0.4*K1)*rand(1,Np);
X(12,:)=0.4*K2+(1*K2-0.4*K2)*rand(1,Np);
X(13,:)=0.4*K3+(1*K3-0.4*K3)*rand(1,Np);
X(14,:)=0.4*K4+(1*K4-0.4*K4)*rand(1,Np);
X(15,:)=0.4*K5+(1*K5-0.4*K5)*rand(1,Np);
%
X(16,:)=0.4*C1+(1*C1-0.4*C1)*rand(1,Np);
X(17,:)=0.4*C2+(1*C2-0.4*C2)*rand(1,Np);
X(18,:)=0.4*C3+(1*C3-0.4*C3)*rand(1,Np);
X(19,:)=0.4*C4+(1*C4-0.4*C4)*rand(1,Np);
X(20,:)=0.4*C5+(1*C5-0.4*C5)*rand(1,Np);

X_s(:,1)=mean(X,2);

%
tic
for i=1:numel(t)-1
    i    % time counter
      
   %-------------------------------------------------------------------
%
    
     for aux=1:1
%
         for j=1:Np
%
           X1(1,j)=X(1,j)+ X(2,j)*dt;
           X1(2,j)=X(2,j)-( (X(11,j)+X(12,j))*X(1,j)- X(12,j)*X(3,j) )*dt;
           X1(2,j)=X1(2,j)-( (X(16,j)+X(17,j))*X(2,j)-X(17,j)*X(4,j) )*dt;
           X1(2,j)=X1(2,j)+ ft*dt;
           X1(2,j)=X1(2,j) + sig_pros*sqrt(dt)*randn;    
%    
           X1(3,j)=X(3,j)+ X(4,j)*dt;
           X1(4,j)=X(4,j)-( (X(12,j)+X(13,j))*X(3,j)- X(13,j)*X(5,j)- X(12,j)*X(1,j) )*dt;
           X1(4,j)=X1(4,j)-( (X(17,j)+X(18,j))*X(4,j)- X(18,j)*X(6,j)- X(17,j)*X(2,j))*dt;
           X1(4,j)=X1(4,j)+ ft*dt;
           X1(4,j)=X1(4,j) + sig_pros*sqrt(dt)*randn;
%    
           X1(5,j)=X(5,j)+ X(6,j)*dt;
           X1(6,j)=X(6,j)-( (X(13,j)+X(14,j))*X(5,j)- X(14,j)*X(7,j)- X(13,j)*X(3,j) )*dt;
           X1(6,j)=X1(6,j)-( (X(18,j)+X(19,j))*X(6,j)- X(19,j)*X(8,j)- X(18,j)*X(4,j))*dt;
           X1(6,j)=X1(6,j)+ ft*dt;
           X1(6,j)=X1(6,j) + sig_pros*sqrt(dt)*randn(1,1);
%   
           X1(7,j)=X(7,j)+ X(8,j)*dt;
           X1(8,j)=X(8,j)-( (X(14,j)+X(15,j))*X(7,j)- X(15,j)*X(9,j)- X(14,j)*X(5,j) )*dt;
           X1(8,j)=X1(8,j)-( (X(19,j)+X(20,j))*X(8,j)- X(20,j)*X(10,j)- X(19,j)*X(6,j))*dt;
           X1(8,j)=X1(8,j)+ ft*dt;
           X1(8,j)=X1(8,j) + sig_pros*sqrt(dt)*randn(1,1);
%    
           X1(9,j)=X(9,j)+ X(10,j)*dt;
           X1(10,j)=X(10,j)-(X(15,j)*X(9,j)- X(15,j)*X(7,j) )*dt;
           X1(10,j)=X1(10,j)-(X(20,j)*X(10,j)- X(20,j)*X(8,j))*dt;
           X1(10,j)=X1(10,j)+ ft*dt;
           X1(10,j)=X1(10,j) + sig_pros*sqrt(dt)*randn(1,1);       
%    
           X1(11,j)=X(11,j)+ sigma_K*sqrt(dt)*randn;
           X1(12,j)=X(12,j)+ sigma_K*sqrt(dt)*randn;
           X1(13,j)=X(13,j)+ sigma_K*sqrt(dt)*randn;
           X1(14,j)=X(14,j)+ sigma_K*sqrt(dt)*randn;        
           X1(15,j)=X(15,j)+ sigma_K*sqrt(dt)*randn;
%                
           X1(16,j)=X(16,j)+ sigma_C*sqrt(dt)*randn;
           X1(17,j)=X(17,j)+ sigma_C*sqrt(dt)*randn;
           X1(18,j)=X(18,j)+ sigma_C*sqrt(dt)*randn;
           X1(19,j)=X(19,j)+ sigma_C*sqrt(dt)*randn;
           X1(20,j)=X(20,j)+ sigma_C*sqrt(dt)*randn;   
%        
         end
%         
         beta_hat=zeros(1,N);
         for u=1:Np 
           beta_hat(u)=beta_hat(u)+1/aux_it*multinorm_pdf(z(i+1),X1(1:2:Index,u),sigma_meas*sqrt(dt)); 
         end
     end
     beta_hat1=beta_hat/sum(beta_hat);%%normalized
     beta_hat_new(:,1)=beta_hat(1,resample(beta_hat1));%%resampled beta hat
     X=X(:,resample(beta_hat1));
     X1_zeta=X1(:,resample(beta_hat1));
 %    
   for j=1:Np
%
    X1(1,j)=X(1,j)+ X(2,j)*dt;
    X1(2,j)=X(2,j)-( (X(11,j)+X(12,j))*X(1,j)- X(12,j)*X(3,j) )*dt;
    X1(2,j)=X1(2,j)-( (X(16,j)+X(17,j))*X(2,j)-X(17,j)*X(4,j) )*dt;
    X1(2,j)=X1(2,j)+ ft*dt;
    X1(2,j)=X1(2,j) + sig_pros*sqrt(dt)*randn;    
%    
    X1(3,j)=X(3,j)+ X(4,j)*dt;
    X1(4,j)=X(4,j)-( (X(12,j)+X(13,j))*X(3,j)- X(13,j)*X(5,j)- X(12,j)*X(1,j) )*dt;
    X1(4,j)=X1(4,j)-( (X(17,j)+X(18,j))*X(4,j)- X(18,j)*X(6,j)- X(17,j)*X(2,j))*dt;
    X1(4,j)=X1(4,j)+ ft*dt;
    X1(4,j)=X1(4,j) + sig_pros*sqrt(dt)*randn;
%    
    X1(5,j)=X(5,j)+ X(6,j)*dt;
    X1(6,j)=X(6,j)-( (X(13,j)+X(14,j))*X(5,j)- X(14,j)*X(7,j)- X(13,j)*X(3,j) )*dt;
    X1(6,j)=X1(6,j)-( (X(18,j)+X(19,j))*X(6,j)- X(19,j)*X(8,j)- X(18,j)*X(4,j))*dt;
    X1(6,j)=X1(6,j)+ ft*dt;
    X1(6,j)=X1(6,j) + sig_pros*sqrt(dt)*randn(1,1);
%    
    X1(7,j)=X(7,j)+ X(8,j)*dt;
    X1(8,j)=X(8,j)-( (X(14,j)+X(15,j))*X(7,j)- X(15,j)*X(9,j)- X(14,j)*X(5,j) )*dt;
    X1(8,j)=X1(8,j)-( (X(19,j)+X(20,j))*X(8,j)- X(20,j)*X(10,j)- X(19,j)*X(6,j))*dt;
    X1(8,j)=X1(8,j)+ ft*dt;
    X1(8,j)=X1(8,j) + sig_pros*sqrt(dt)*randn(1,1);
 %   
    X1(9,j)=X(9,j)+ X(10,j)*dt;
    X1(10,j)=X(10,j)-(X(15,j)*X(9,j)- X(15,j)*X(7,j) )*dt;
    X1(10,j)=X1(10,j)-(X(20,j)*X(10,j)- X(20,j)*X(8,j))*dt;
    X1(10,j)=X1(10,j)+ ft*dt;
    X1(10,j)=X1(10,j) + sig_pros*sqrt(dt)*randn(1,1);       
 %   
        X1(11,j)=X(11,j)+ sigma_K*sqrt(dt)*randn;
        X1(12,j)=X(12,j)+ sigma_K*sqrt(dt)*randn;
        X1(13,j)=X(13,j)+ sigma_K*sqrt(dt)*randn;
        X1(14,j)=X(14,j)+ sigma_K*sqrt(dt)*randn;        
        X1(15,j)=X(15,j)+ sigma_K*sqrt(dt)*randn;
 %               
        X1(16,j)=X(16,j)+ sigma_C*sqrt(dt)*randn;
        X1(17,j)=X(17,j)+ sigma_C*sqrt(dt)*randn;
        X1(18,j)=X(18,j)+ sigma_C*sqrt(dt)*randn;
        X1(19,j)=X(19,j)+ sigma_C*sqrt(dt)*randn;
        X1(20,j)=X(20,j)+ sigma_C*sqrt(dt)*randn;

     end
%
     weight_sum=0;
     for vi=1:N
% 
         weight_m(1,vi)=multinorm_pdf(z(i+1),X1(1:2:Index,vi),sigma_meas*sqrt(dt));
         New_wt(1,vi)=multinorm_pdf(z(i+1),X1_zeta(1:2:Index,vi),sigma_meas*sqrt(dt));
         weight(1,vi)=weight_m(1,vi)/New_wt(1,vi);
         weight_sum=weight_sum+weight(1,vi);
%
     end
%     
     weight(1,:)=weight(1,:)/weight_sum(1);
     new_weight = resample(weight);   %% resampling
%
     X2= X1(:,new_weight);
%  
    X(1:20,:)=X2(1:20,:);
    X_s(:,i+1)=mean(X,2);
% 
end
%
       figure(3)    % Figure 6.12c
        plot(t,2101200*K1*ones(1,size(t,2)),'black'); hold on 
        plot(t,2101200*X_s(11,:),'black'); hold on 
        plot(t,2101200*X_s(12,:),'black'); hold on 
        plot(t,2101200*X_s(13,:),'black'); hold on 
        plot(t,2101200*X_s(14,:),'black'); hold on 
        plot(t,2101200*X_s(15,:),'black'); hold on 
%        
        figure(4)    % Figure 6.12d
        plot(t,2101200*C1*ones(1,size(t,2)),'black'); hold on 
        plot(t,2101200*X_s(16,:),'black'); hold on 
        plot(t,2101200*X_s(17,:),'black'); hold on 
        plot(t,2101200*X_s(18,:),'black'); hold on        
        plot(t,2101200*X_s(19,:),'black'); hold on 
        plot(t,2101200*X_s(20,:),'black'); hold on 

