



%                         Van der Pol oscillator   by SA
%                    
%                       Figure 6.5c  and  Figures 6.6 ef                   
%
% 
% function [minimum,fval] = anneal(loss, parent, options)
% function [minimum,fval] = anneal(loss,parent, options)
% parent
 
% ANNEAL  Minimizes a function with the method of simulated annealing
% (Kirkpatrick et al., 1983)
%
%  ANNEAL takes three input parameters, in this order:
%
%  LOSS is a function handle (anonymous function or inline) with a loss
%  function, which may be of any type, and needn't be continuous. It does,
%  however, need to return a single value.
%
%  PARENT is a vector with initial guess parameters. You must input an
%  initial guess.
%
%  OPTIONS is a structure with settings for the simulated annealing. If no
%  OPTIONS structure is provided, ANNEAL uses a default structure. OPTIONS
%  can contain any or all of the following fields (missing fields are
%  filled with default values):
%
%       Verbosity: Controls output to the screen.
%                  0 suppresses all output
%                  1 gives final report only [default]
%                  2 gives temperature changes and final report 
%       Generator: Generates a new solution from an old one.
%                  Any function handle that takes a solution as input and
%                  gives a valid solution (i.e. some point in the solution
%                  space) as output.
%                  The default function generates a row vector which
%                  slightly differs from the input vector in one element:
%                  @(x) (x+(randperm(length(x))==length(x))*randn/100)
%                  Other examples of possible solution generators:
%                  @(x) (rand(3,1)): Picks a random point in the unit cube
%                  @(x) (ceil([9 5].*rand(2,1))): Picks a point in a 9-by-5
%                                                 discrete grid
%        InitTemp: The initial temperature, can be any positive number.
%                  Default is 1.
%        StopTemp: Temperature at which to stop, can be any positive number
%                  smaller than InitTemp. 
%                  Default is 1e-8.
%         StopVal: Value at which to stop immediately, can be any output of
%                  LOSS that is sufficiently low for you.
%                  Default is -Inf.
%       CoolSched: Generates a new temperature from the previous one.
%                  Any function handle that takes a scalar as input and
%                  returns a smaller but positive scalar as output. 
%                  Default is @(T) (.8*T)
%      MaxConsRej: Maximum number of consecutive rejections, can be any
%                  positive number.
%                  Default is 1000.
%        MaxTries: Maximum number of tries within one temperature, can be
%                  any positive number.
%                  Default is 300.
%      MaxSuccess: Maximum number of successes within one temperature, can
%                  be any positive number.
%                  Default is 20.
%
%
%  Usage:
%     [MINIMUM,FVAL] = ANNEAL(LOSS,minF,[OPTIONS]);
%          MINIMUM is the solution which generated the smallest encountered
%          value when input into LOSS.
%          FVAL is the value of the LOSS function evaluated at MINIMUM.
%     OPTIONS = ANNEAL();
%          OPTIONS is the default options structure.
%
%
%  Example:
%     The so-called "six-hump camelback" function has several local minima
%     in the range -3<=x<=3 and -2<=y<=2. It has two global minima, namely
%     f(-0.0898,0.7126) = f(0.0898,-0.7126) = -1.0316. We can define and
%     minimise it as follows:
%          camel = @(x,y)(4-2.1*x.^2+x.^4/3).*x.^2+x.*y+4*(y.^2-1).*y.^2;
%          loss = @(p)camel(p(1),p(2));
%          [x f] = ANNEAL(loss,[0 0])
%     We get output:
%               Initial temperature:     	1
%               Final temperature:       	3.21388e-007
%               Consecutive rejections:  	1027
%               Number of function calls:	6220
%               Total final loss:        	-1.03163
%               x =
%                  -0.0899    0.7127
%               f =
%                  -1.0316
%     Which reasonably approximates the analytical global minimum (note
%     that due to randomness, your results will likely not be exactly the
%     same).

%  Reference:
%    Kirkpatrick, S., Gelatt, C.D., & Vecchi, M.P. (1983). Optimization by
%    Simulated Annealing. _Science, 220_, 671-680.

%   joachim.vandekerckhove@psy.kuleuven.be
%   $Revision: v5 $  $Date: 2006/04/26 12:54:04 $
%'Generator',@(x) (x+(randperm(length(x))==length(x))*randn/20),...
% def = struct(...
%         'CoolSched',@(T) (.9*T),...   %0.8T changed to 0.9T
%         'Generator',@(x) (x+(randperm(length(x))==length(x))*randn/100),...
%         'InitTemp',5,...
%         'MaxConsRej',1000,...
%         'MaxSuccess',20,...
%         'MaxTries',300,...
%         'StopTemp',1e-8,...
%         'StopVal',-Inf,...
%         'Verbosity',2);



%       Fig6_5c_6_6ef_SA_vanderpol.m
%---------------------------------------------------------
%      parameter identification of Van der Pol oscillator  by GA
%
%      Figure 6.4  - referenec (observed solution)
%      Figure 6.5(c) - evolution of the objective function with iterations
%      Figure 6.6 (e&f) - evolution of the damping (c) and stiffness (k)
%                    parameters with terations
%-------------------------------------------------------------------------

%  routines called : vanderpol_states.m 
%                    fun_vanderpol.m
%
%


clc;
close all;
clear all;

global z eta stif

% following are the reference values for c (variable 'eta') 
%         and k (variable 'stif') to get the reference (observed) solution
eta=4;
stif = 10;

InitTemp=5;

Time=5.0;
dtt=0.01;
nTt=Time/dtt+1;
tt=linspace(0,Time,nTt)'; % time variables



% call to obtain the reference (observed) solution 
%           for x1(t) % x2(t) (Figure 6.4)  
[z]=vanderpol_states();
figure(5)
plot(tt,z(1,:)),hold on
% figure(6)
plot(tt,z(2,:))


% bounds for the two parameters
varhi(1)=10;
varhi(2)=20;
varlo(1)=1;
varlo(2)=5;

% Check input
% if ~nargin %user wants default options, give it and stop
%     minimum = def;
%     return
% elseif nargin<2, %user gave only objective function, throw error
%     error('MATLAB:anneal:noParent','You need to input a first guess.');
% elseif nargin<3, %user gave no options structure, use default
%     options=def;
% else %user gave all input, check if options structure is complete
%     if ~isstruct(options)
%         error('MATLAB:anneal:badOptions',...
%             'Input argument ''options'' is not a structure')
%     end
%     fs = {'CoolSched','Generator','InitTemp','MaxConsRej',...
%         'MaxSuccess','MaxTries','StopTemp','StopVal','Verbosity'};
%     for nm=1:length(fs)
%         if ~isfield(options,fs{nm}), options.(fs{nm}) = def.(fs{nm}); end
%     end
% end

% main settings
% newsol = options.Generator;      % neighborhood space function
% Tinit = options.InitTemp;        % initial temp
% minT = options.StopTemp;         % stopping temp
% cool = options.CoolSched;        % annealing schedule
% minF = options.StopVal;
% max_consec_rejections = options.MaxConsRej;
% max_try = options.MaxTries;
% max_success = options.MaxSuccess;
% report = options.Verbosity;
k = 1;                           % boltzmann constant

% counters etc
 Tinit=5;
itry = 1;
success = 0;
finished = 0;
consec = 0;
T = Tinit;
parent(1)=1;
parent(2)=1;

% Fitness values for the initial population
initenergy = fun_vanderpol(parent);
oldenergy = initenergy

total = 0;
 fprintf(1,'\n  T = %7.5f, loss = %10.5f\n',T,oldenergy);



isuc=1;
temp(isuc)=T;
opt(isuc)=oldenergy;
x1(isuc)=parent(1);
x2(isuc)=parent(2);
minT=1e-15;
minF=1e-5;
max_try=1000;
max_success=1000;
max_consec_rejections=50000;
tic

while itry<max_try;
    itry = itry+1; % increase the iteration counter
    current = parent; 
    
  % decrease T according to the cooling (annealing) schedule
    if itry >= max_try || success >= max_success;
        if T < minT || consec >= max_consec_rejections;
            finished = 1;
            total = total + itry;
            break;
        else
%             T = cool(T);  % decrease T according to cooling schedule
             T = 0.8*T;  % decrease T according to cooling schedule
%             if report==2, % output
                fprintf(1,'  T = %7.5f, loss = %10.5f\n',T,oldenergy);
                
                isuc=isuc+1
                opt(isuc)=oldenergy
                x1(isuc)=parent(1)
                x2(isuc)=parent(2)
                temp(isuc)=T;
%             end
            total = total + itry;
            itry = 1;
            success = 1;
        end
    end
    
%     newparam = newsol(current);
    newparam(1)=(varhi(1)-varlo(1))*rand+varlo(1);
     newparam(2)=(varhi(2)-varlo(2))*rand+varlo(2);
%     newenergy = loss(newparam);
     newenergy = fun_vanderpol(newparam);
    newparam;
    if (newenergy < minF),
        parent = newparam; 
        oldenergy = newenergy;
        break
    end
    
    if (oldenergy-newenergy > 1e-6)
        parent = newparam;
        oldenergy = newenergy;
        success = success+1;
        consec = 0;
    else
        if (rand < exp( (oldenergy-newenergy)/(k*T) ));
            parent = newparam;
            oldenergy = newenergy;
            success = success+1;
        else
            consec = consec+1;
        end
    end
%     iters(itry)=itry;
%     opt(itry)=oldenergy;
end
toc
minimum = parent;
fval = oldenergy;

% if report;
    fprintf(1, '\n  Initial temperature:     \t%g\n', Tinit);
    fprintf(1, '  Final temperature:       \t%g\n', T);
    fprintf(1, '  Consecutive rejections:  \t%i\n', consec);
    fprintf(1, '  Number of function calls:\t%i\n', total);
    fprintf(1, '  Total final loss:        \t%g\n', fval);
% end

 iters=(1:isuc)
% size(iters)
% size(opt)

figure(1)
plot(iters,opt)
figure(2)
plot(iters,temp)
figure(3)
plot(iters,x1)
figure(4)
plot(iters,x2)
 iters
 opt











